#ifndef _APFO_H
#define _APFO_H

/**
 */
/*
 * (c) Copyright 2014, 2015 Hewlett-Packard Development Company, L.P
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor
 * Boston, MA 02110-1301, USA.
 */

extern char *upper_version_date;
extern char *resolve_version_date;
extern char *lower_version_date;
extern int pfo_debug_flag;

#define MAX_PATH_NUMBER 10000
#define MAX_FDEV_NUMBER 10000
#define MAX_PATH_LEN 17
#define FDEV_MUTEX_COUNT (MAX_FDEV_NUMBER+4)
#define BALANCE_MUTEX (MAX_FDEV_NUMBER+3)
#define STRUCT_MUTEX (MAX_FDEV_NUMBER+2)
#define OPEN_MUTEX (MAX_FDEV_NUMBER+1)

#define SN_SIZE 32

#define WWN_SIZE 8

/* Return values from lower_send() */
#define PFO_GOOD 0
#define PFO_PATH_DOWN 1
#define PFO_CMD_TIMEOUT 2
#define PFO_CMD_ERROR 3
#define PFO_OTHER_ERROR 5
#define PFO_FATAL_ERROR 6

/* Load balancing operation levels */
#define FULL_BALANCING 2
#define LESS_BALANCING 1
#define NO_BALANCING 0
extern int pfo_balance;
/* Return generic routine results */
#define PFO_ERROR 1000

/* Return values from decide_add_new_path() */
#define PFO_NOT_APFO_DEVICE 100
#define PFO_OLD_APFO_DEVICE 101
#define PFO_NEW_APFO_DEVICE 102

/* Return values from lower_remove_path() */
#define PFO_NOT_APFO_PATH (-1)

/* This structure is used to pass SCSI commands
 * from upper.c through decide.c to lower.c,
 * and back with errors.
 */
struct apfo_scsi_command {
/* OS independent items below here */
	unsigned char *scsi_cmd;
	int cmd_len;
	unsigned int scsi_status_code;
	unsigned int sense_key;
	unsigned int asc;
	unsigned int ascq;
	void *buffer;
	unsigned int bufflen;
	int source;
/* OS dependent items below here.
 * These values pass-thru the decision layer.
 * The decision layer should avoid attempting to
 * interpret the meanings of these numbers */
	/* The following are needed before a command is executed */
	int data_direction;
	int timeout;
	int retries;
	int iov_count;
	unsigned int flags;
	void *mdata;
	unsigned long offset;
	/* The following are returned after a command is completed */
	int errors;
	int resid_len;
	unsigned char *raw_sense;
	int sense_len;
	unsigned int response_code;
};

/* Definitions of functions provided by lower.c */
int lower_send(int path, struct apfo_scsi_command *cmd);
int lower_path_status(int path);
/** lower.c - Instantiate a new path to the given structure,
 * return the new path number */
int lower_add_new_path(void *real_sdev);
void lower_remove_path(int path_num);
int lower_print_path(char *string, int path);
void lower_exit(void);
int lower_physical_path_number(int path_num);
int lower_sfmp_send(int path, struct apfo_scsi_command *cmd);

/* Definitions of functions provided by resolve.c */
int resolve_send(int fdev, struct apfo_scsi_command *cmd);
int resolve_open(int fdev, struct file *filp);
int resolve_close(int fdev, struct file *filp);
int resolve_add_new_path(int path, int *fdev,
			unsigned char *wwnn, unsigned char *serial);
int resolve_remove_path(int path_num);
int resolve_print_device(int fdev, char *string, int size);
int resolve_print_sessions(int fdev, char *string, int size);
int resolve_ctrl(int fdev, const char *string, size_t count);
int resolve_ctrl_get(char *string, size_t count);
void resolve_exit(void);
int resolve_get_wwnn_from_path(int path_num, unsigned char wwnn[]);
void resolve_set_last_path(int fdev_num, int path_num);
void resolve_set_is_not_sfmp_device(int fdev_num);
void resolve_not_fo_device(void *pdev, int fdev_num);
void resolve_remove_non_fo_device(int fdev_num);

/* Definitions of functions provided by intercept.c */
/** intercept.c - Receive notification whenever a given device chooses
 * to use the given path */
void intercept_notify_path_change(int fdev, int path);
int intercept_sfmp_open(int fdev_num, struct file *filp);
int intercept_sfmp_close(int fdev_num, struct file *filp);
int intercept_sfmp_send(int fdev_num,
	unchar* cdb,
	unchar cmd_length,
	unchar* buffer,
	uint buffer_length,
	uint data_direction,
	uint timeout,
	unchar* sense,
	int* sense_length,
	int* resid,
	int* result);
int intercept_not_fo_device(void *pdev, int fdev_num, int path_num);

/* Definitions of functions provided by fo_util.c */

/* Definitions of functions provided by upper.c */
int upper_add_new_device(void *replicate_sdev, int fdev_num);
void upper_exit(void);
void upper_not_fo_device(int fdev_num);

/* Constants used for Tracing */
#define PFO_TRACE_BUF_SIZE 4096
#define PFO_TRACE_EMPTY 0
#define PFO_TRACE_ERROR 1
#define PFO_TRACE_COMMAND 2
#define PFO_TRACE_PATH_BROKEN 3
#define PFO_TRACE_SENSE 5
#define PFO_TRACE_ATTEMPT 9
#define PFO_TRACE_UNACCEPTABLE 99
#define PFO_TRACE_PATH_BAD 11
#define PFO_TRACE_PATH_NEW 12
#define PFO_TRACE_DEVICE_NEW 13
#define PFO_TRACE_FATAL 21
#define PFO_TRACE_BREAK_TIMEOUT 22

/* Where this command originated */
#define PFO_SOURCE_ST 201
#define PFO_SOURCE_SG 202
#define PFO_SOURCE_PFO 203
#define PFO_SOURCE_IOCTL 204
#define PFO_SOURCE_CMD_1 205
#define PFO_SOURCE_EXEC 206

/* Definitions of functions provided by trace.c */
void trace_event(unsigned char type, unsigned char source,
			int fdev, int path, void *data);
int trace_get(char *buf);
void trace_exit(void);

/* Definitions of functions provided by fo_util.c */
int check_string_for_valid_path(char *string);
unsigned char random_byte(void);
void print_log(char *string);
int lock_apfo_path(int path_num);
void unlock_apfo_path(int path_num);
int lock_apfo_device(int fdev_num);
void unlock_apfo_device(int fdev_num);
void sleep_half_second(void);
void sleep_six_seconds(void);

#define __BASEFILE (strrchr(__FILE__, '/') ? strrchr(__FILE__, '/') + 1 : strrchr(__FILE__, '\\') ? strrchr(__FILE__, '\\') + 1 : __FILE__)

#define DBG_DEBUG(fmt, args...) debug(__BASEFILE, __LINE__, __func__, 8, fmt, ## args)
#define DBG_INFO(fmt, args...) debug(__BASEFILE, __LINE__, __func__, 4, fmt, ## args)
#define DBG_WARNING(fmt, args...) debug(__BASEFILE, __LINE__, __func__, 2, fmt, ## args)
#define DBG_ERROR(fmt, args...) debug(__BASEFILE, __LINE__, __func__, 1, fmt, ## args)

void debug(const char *, int, const char *, int, const char *, ...);
int init_debug(struct device_driver *pfo_gendrv);
void stop_debug(struct device_driver *pfo_gendrv);

#endif
