/*******************************************************************************
 *   lin_tape_extra_ioctl.h
 *
 *   (C) Copyright International Business Machines  Corp., 2007-2020
 *   This file is part of the IBM Linux Enhanced SCSI Tape and
 *   Medium Changer Device Driver.
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ******************************************************************************/


#ifndef _LIN_TAPE_EXTRA_IOCTL_H_
#define _LIN_TAPE_EXTRA_IOCTL_H_

#include "lin_tape_scsi_ioctl.h"

#include <linux/types.h>

#ifndef uchar
#define uchar unsigned char
#endif

extern int lto_barcode;
extern int ibm3592_barcode;

#define  DEVTYPE_ANY       998
#define  DEVTYPE_LTO        21
#define  DEVTYPE_3592       25

#define LB_BARCODE_LEN      36 /* total length of possible bar code     */
#define DD_SCSI_MAX_SENSE  255 /* maximum length for return error sense */
#define DD_MAX_PRODUCT_ID   16
#define DD_MAX_VENDOR_ID     8
#define DD_MAX_REV_LEVEL     4
#define DD_MAX_LOG_SENSE  1024
#define DD_MAX_TAPEALERT    64

#define DD_MAX_DEVICE_WWNAME      8
#define DD_MAX_DEVICE_SERIAL     36
#define DD_DRIVER_LEVEL_LENGTH   10

/********************************************************************
 *                                                                  *
 * ioctl command definitions                                        *
 *                                                                  *
 ********************************************************************/
#define DDIOC_INQUIRY                 _IOR('d',1,dd_old_inquiry_t)
#define DDIOC_RETURN_ERR              _IOR('d',2,dd_return_error_state_t)

#define DDIOC_LOG_SENSE               _IOR('d',9,dd_log_sense_page_t)
#define DDIOC_XINQUIRY                _IOR('d',10,dd_inquiry_t)
#define DDIOC_QUERY_PR_KEY            _IOR('d',15,dd_query_prkey_t)
#define DDIOC_SET_RESERVE_KEY         _IOR('d',16,dd_set_reserve_key_t)
#define DDIOC_CLEAR_RESERVE_KEY       _IOR('d',17,dd_clear_reserve_t)
#define DDIOC_PREEMP_ABORT            _IOR('d',18,dd_preempt_abort_t)

#define LBIOC_RETURN_ELEMENT_COUNT    _IOR('l',1,lb_element_count_t)

#define LBIOC_LIBRARY_INVENTORY       _IOWR('l',2,lb_inventory_t)

#define LBIOC_MOVE_MEDIUM             _IOWR('l',3,lb_move_medium_t)
#define LBIOC_LIBRARY_AUDIT           _IOWR('l',4,lb_audit_t)
#define LBIOC_EXTEND_RETRACT_IO_PT    _IOWR('l',5,lb_extend_retract_t)
#define LBIOC_LIBRARY_INFO            _IOR('l',6,lb_library_info_t)
#define LBIOC_POSITION                _IOWR('l',7,lb_position_t)

#define LBIOC_XDRIVE_INVENTORY        _IOWR('l',9,lb_Xdrive_inventory_t)
#define LBIOC_ENH_LIBRARY_INVENTORY   _IOWR('l',11,lb_enh_inventory_t)


typedef enum erp_compcode_e {
	DD_UNKNOWN_ERROR = -1,   /*  -1 */
	DD_NO_ERROR = 0,         /*   0 */
	DD_NO_SENSE = 200,       /* 200 */
	DD_DEVICE_DRIVER_FAILURE,/* 201 */
	DD_EEPROM_FAILURE,       /* 202 */
	DD_MANUAL_INTERVENTION,  /* 203 */
	DD_RECOVERED_ERROR,      /* 204 */
	DD_SCSI_ADAPTER_ERROR,   /* 205 */
	DD_SCSI_ERROR,           /* 206 */
	DD_ILLEGAL_REQUEST,      /* 207 */
	DD_COMMAND_ABORTED,      /* 208 */
	DD_HARDWARE_MICROCODE,   /* 209 */
	DD_UNIT_ATTENTION,       /* 210 */
	DD_SCSI_BUSY,            /* 211 */
	DD_PR_NOT_SUPPORTED,     /* 212 */
	DD_PR_OP_FAILURE,        /* 213 */

/*
 * completion code values for media changer
 */
	DD_CARTRIDGE_ENTRY_EXIT = 300,  /* 300 */
	DD_CARTRIDGE_LOAD_FAILURE,      /* 301 */
	DD_CARTRIDGE_IN_FAILED_DRIVE,   /* 302 */
	DD_CAROUSEL_NOT_LOADED,         /* 303 */
	DD_CHANGER_FAILURE,             /* 304 */
	DD_DRIVE_FAILURE,               /* 305 */
	DD_DRIVE_OR_MEDIA_FAILURE,      /* 306 */
	DD_ENTRY_EXIT_FAILURE,          /* 307 */
	DD_ENTRY_EXIT_NOT_PRESENT,      /* 308 */
	DD_LIBRARY_AUDIT,               /* 309 */
	DD_LIBRARY_FULL,                /* 310 */
	DD_MEDIA_EXPORT,                /* 311 */
	DD_SLOT_FAILURE,                /* 312 */
	DD_SLOT_OR_MEDIA_FAILURE,       /* 313 */
	DD_SOURCE_EMPTY,                /* 314 */
	DD_DESTINATION_FULL,            /* 315 */
	DD_CLEANER_INST,                /* 316 */
	DD_MEDIA_NOT_EJECTED,           /* 317 */
	DD_IOPORT_NOT_CONFIG,           /* 318 */
	DD_FIRST_DEST_EMPTY,            /* 319 */
	DD_NO_INVENTORY_INFO,           /* 320 */
	DD_RES_MISMATCH,                /* 321 */
	DD_INIT_RANGE_FAILED,           /* 322 */

/*
 * completion code values for tape and optical drives
 */
	DD_END_PHYSICAL_MEDIA = 400,    /* 400 */
	DD_MEDIA_BLANK,                 /* 401 */
	DD_MEDIA_CORRUPTED,             /* 402 */
	DD_MEDIA_FAILURE,               /* 403 */
	DD_MEDIA_INCOMPATIBILITY,       /* 404 */
	DD_SECTOR_RELOCATION,           /* 405 */
	DD_SECTOR_OUT_OF_RANGE,         /* 406 */
	DD_WRITE_PROTECT,               /* 407 */
	DD_CLEAN_MEDIA,                 /* 408 */
	DD_MEDIA_FAULT,                 /* 409 */
	DD_CLEANING_COMPLETE,           /* 410 */
	DD_LOGICAL_END_OF_MEDIA,        /* 411 */
	DD_MEDIA_NOT_PRESENT,           /* 412 */
	DD_BEGINNING_OF_MEDIA,          /* 413 */
	DD_ERASE_FAILURE,               /* 414 */
	DD_WRITE_TO_WRITTEN_WORM,       /* 415 */
	DD_WRONG_LENGTH_BLOCK,          /* 416 */
	DD_OPEN_READ_ONLY,              /* 417 */
	DD_OPEN_WRITE_ONLY,             /* 418 */
	DD_VOLUME_FENCED,               /* 419 */
	DD_LOGICAL_WRITE_PROTECT,       /* 420 */
	DD_TAPE_ALERT_PENDING,          /* 421 */
	DD_CLEANING_REQUIRED            /* 422 */
} erp_compcode_t;

/*
 * LBIOC_RETURN_ELEMENT_COUNT definitions
 */
typedef struct {
	int32_t compcode;       /* O - request completion code            */
	int32_t drives;         /* O - number of drives                   */
	int32_t slots;          /* O - number of storage slots            */
	int32_t transp_elems;   /* O - number of media transport elements */
	int32_t eeports;        /* O - number of import/export elements   */
} lb_element_count_t;

/*
 * LBIOC_LIBRARY_INVENTORY definitions
 */

/* Data Transfer Element Descriptor (Drive) */
typedef struct {                /* This entire structure is OUTPUT INFO       */
	ushort address;         /* Element address of drive                   */
	int32_t abnormal_state; /* 0 is normal state. Negative otherwise      */
	uchar accessible;       /* True if accessible by the CHM, else False  */
	uchar full;             /* True if it contains media, else False      */
	uchar scsi_bus_not_pres;/* True if drive bus differs from robot bus   */
	uchar scsi_bus_id_flag; /* True if scsi bus id is valid               */
	uchar scsi_bus_id;      /* scsi bus id                                */
	uchar lun_flag;         /* True if logical unit number valid          */
	uchar lun;              /* logical unit nunber                        */
	uchar source_addr_flag; /* True if src st ad and invert media valid   */
	ushort source_elem_addr;/* source storage element address             */
	uchar invert_media_flag;/* True if media in this element was inverted */
	uchar barcode_flag;     /* True if primary voltag(barcode) is valid   */
	int32_t barcode_len;    /* Length in bytes of valid barcode           */
	uchar barcode[LB_BARCODE_LEN]; /* barcode label info                  */
} lb_drive_state_t;

/* Storage Element Descriptor */
typedef struct {                /* This entire structure is OUTPUT            */
	ushort address;         /* Element address of storage slot            */
	int32_t abnormal_state; /* 0 if drive is in normal state              */
	uchar accessible;       /* True if accessible by CHM, False otherwise */
	uchar full;             /* True if it contains media. False otherwise */
	uchar source_addr_flag; /* True if src st ad and invert media valid   */
	ushort source_elem_addr;/* source storage element address             */
	uchar invert_media_flag;/* True if media in this element was inverted */
	uchar barcode_flag;     /* True if primary voltag(barcode) is valid   */
	int32_t barcode_len;    /* Length in bytes of valid barcode           */
	uchar barcode[LB_BARCODE_LEN];  /* barcode label info                 */
} lb_storage_element_state_t;

/* Import/Export Element Descriptor */
typedef struct {                /* This entire structure is OUTPUT            */
	ushort address;         /* element address of entry exit port         */
	uchar import_enable;    /* True if cartridge can be imported          */
	uchar export_enable;    /* True if cartridge can be exported          */
	int32_t abnormal_state; /* 0 if drive is in normal state              */
	uchar accessible;       /* True if accessible by the CHM, else False  */
	uchar full;             /* True if it contains media, else False      */
	uchar oper_placed_media;/* True if operator placed the media          */
	uchar source_addr_flag; /* True if src st ad and invert media valid   */
	ushort source_elem_addr;/* source storage element address             */
	uchar invert_media_flag;/* True if media in this element was inverted */
	uchar barcode_flag;     /* True if primary voltag is valid            */
	int32_t barcode_len;    /* Length in bytes of valid barcode           */
	uchar barcode[LB_BARCODE_LEN];  /* barcode label info                 */
} lb_import_export_state_t;

/* Medium changer Element Descriptor */
typedef struct {                /* This entire structure is OUTPUT            */
	ushort address;         /* Element address of the medium changer      */
	int32_t abnormal_state; /* 0 if drive is in normal state.             */
	uchar accessible;       /* True if accessible by the CHM, else False  */
	uchar full;             /* True if it contains media, else False      */
	uchar source_addr_flag; /* True if src st ad and invert media valid   */
	ushort source_elem_addr;/* source storage element address             */
	uchar invert_media_flag;/* True if media in this element was inverted */
	uchar barcode_flag;     /* True if primary voltag is valid            */
	int32_t barcode_len;    /* Length in bytes of valid barcode           */
	uchar barcode[LB_BARCODE_LEN];  /* barcode label info                 */
} lb_medium_changer_state_t;

/* Values for abnormal_state field above */
#define LB_ABNORMAL_IGNORE             0
#define LB_FULL_STATUS_UNSURE          0x00000001
#define LB_BAR_CODE_LABEL_UNSURE       0x00000002
#define LB_CART_HOLDER_NOT_PRESENT     0x00000004
#define LB_CTS_NOT_INSTALLED           0x00000008
#define LB_CTS_NOT_TARGETED            0x00000010
#define LB_CTS_HAS_TARGET_BLOCK        0x00000020
#define LB_CAN_NOT_READ_LABEL          0x00000040
#define LB_CTS_DOOR_CLOSED             0x00000080
#define LB_UNABLE_TO_CALIBRATE         0x00000100
#define LB_CLEANER_INSTALLED           0x00000200
#define LB_OPER_REMOVAL_REQUEST        0x00000400
#define LB_ELEMENT_RESERVED            0x00000800
#define LB_DRIVE_NOT_PRESENT           0x00001000
#define LB_DRIVE_REQUIRES_CLEANING     0x00002000
#define LB_DRIVE_ERROR                 0x00004000
#define LB_TAPE_IN_DRIVE               0x00008000
#define LB_WARNING                     0x00010000
#define LB_COMMUNICATION_ERROR         0x00020000
#define LB_CLEANING_ERROR              0x00040000
#define LB_MEDIUM_NOT_PRESENT          0x00080000
#define LB_MAGAZINE_ERROR              0x00100000
#define LB_CONDITIONS_CHANGED          0x00200000
#define LB_MEDIUM_REMOVAL_PREVENTED    0x00400000
#define LB_INVALID_INFO                0x80000000
#define LB_CELL_TYPE_MISMATCH          0x00800000

typedef struct {
	int32_t compcode;       /* O - completion code                        */
	int32_t drives;         /* I - number of drives                       */
	int32_t slots;          /* I - number of storage slots                */
	int32_t eeports;        /* I - number of import/export elemnts        */
	int32_t transp_elems;   /* I - number of transport elements           */
	lb_drive_state_t* drive_state;  /* O - Pointer to user buffer         */
	lb_storage_element_state_t* storage_element_state; /* O - user buffer */
	lb_import_export_state_t* import_export_state;     /* O - user buffer */
	lb_medium_changer_state_t* medium_changer_state;   /* O - user buffer */
} lb_inventory_t;

/*
 * LBIOC_ENH_LIBRARY_INVENTORY definitions
 */

/* Enhanced Storage Element(slot) Descriptor */
typedef struct {                /* This entire structure is OUTPUT            */
	ushort address;         /* Element address of storage slot            */
	int32_t abnormal_state; /* 0 if drive is in normal state              */
	uchar accessible;       /* True if accessible by CHM, else False      */
	uchar full;             /* True if it contains media, else False      */
	uchar source_addr_flag; /* True if src st ad and invert media valid   */
	ushort source_elem_addr;/* source storage element address             */
	uchar invert_media_flag;/* True if media in this element was inverted */
	uchar barcode_flag;     /* True if primary voltag is valid            */
	int32_t barcode_len;    /* Length in bytes of valid barcode           */
	uchar barcode[LB_BARCODE_LEN];  /* barcode label info                 */
	int32_t media_devtype;  /* the PVR Device type.Family of media        */
	int32_t spec_media_type;/* MT_XXX value.0xFFFFFFFF if unknown         */
	int32_t element_devtype;/* element domain. What media can fill slot   */
	uchar worm;             /* True if media is WORM                      */
	uchar reserverd[15];
} lb_enh_storage_element_state_t;

/* Enhanced Import/Export Element Descriptor */
typedef struct {                /* This entire structure is OUTPUT            */
	ushort address;         /* element address of entry exit port         */
	uchar import_enable;    /* True if cartridge can be imported          */
	uchar export_enable;    /* True if cartridge can be exported          */
	int32_t abnormal_state; /* 0 if drive is in normal state.             */
	uchar accessible;       /* True if accessible by the CHM.             */
	uchar full;             /* True if it contains media, else False      */
	uchar oper_placed_media;/* True if operator placed the media          */
	uchar source_addr_flag; /* True if src st ad and invert media valid   */
	ushort source_elem_addr;/* source storage element address             */
	uchar invert_media_flag;/* True if media in this element was inverted */
	uchar barcode_flag;     /* True if primary voltag is valid            */
	int32_t barcode_len;    /* Length in bytes of valid barcode           */
	uchar barcode[LB_BARCODE_LEN];  /* barcode label info                 */
	int32_t media_devtype;  /* the PVR Device type.Family of media        */
	int32_t spec_media_type;/* MT_XXX value.0xFFFFFFFF if unknown         */
	int32_t element_devtype;/* element domain. What media can fill slot   */
	uchar worm;             /* True if media is WORM                      */
	uchar reserverd[15];
} lb_enh_import_export_state_t;

/* Enhanced Medium Changer Element Descriptor */
typedef struct {                /* This entire structure is OUTPUT            */
	ushort address;         /* Element address of the medium changer      */
	int32_t abnormal_state; /* 0 if drive is in normal state.             */
	uchar accessible;       /* T/F - Accessible by the CHM?               */
	uchar full;             /* True if it contains media. False otherwise */
	uchar source_addr_flag; /* True if src st ad and invert media valid   */
	ushort source_elem_addr;/* source storage element address             */
	uchar invert_media_flag;/* True if media in this element was inverted */
	uchar barcode_flag;     /* True if primary voltag is valid            */
	int32_t barcode_len;    /* Length in bytes of valid barcode           */
	uchar barcode[LB_BARCODE_LEN];  /* barcode label info                 */
	int32_t media_devtype;  /* the PVR Device type.Family of media        */
	int32_t spec_media_type;/* MT_XXX value.0xFFFFFFFF if unknown         */
	int32_t element_devtype;/* element domain. What media can fill slot   */
	int32_t reserverd[4];
} lb_enh_medium_changer_state_t;

/* Enhanced Data Transfer Element Descriptor (Drive) */
typedef struct {                 /* This entire structure is OUTPUT           */
	ushort address;          /* Element address of drive                  */
	int32_t abnormal_state;  /* 0 if drive is in normal state.            */
	uchar accessible;        /* T/F - Accessible by the CHM?              */
	uchar full;              /* True if it contains media, else False     */
	uchar scsi_bus_not_pres; /* True if drive bus differs from robot bus  */
	uchar scsi_bus_id_flag;  /* True if scsi bus id is valid              */
	uchar scsi_bus_id;       /* scsi bus id                               */
	uchar lun_flag;          /* True if logical unit number valid         */
	uchar lun;               /* logical unit number                       */
	uchar source_addr_flag;  /* True if src st ad and invert media valid  */
	ushort source_elem_addr; /* source storage element address            */
	uchar invert_media_flag; /* True if media in this element was inverted*/
	uchar barcode_flag;      /* True if primary voltag is valid           */
	int32_t barcode_len;     /* Length in bytes of valid barcode          */
	uchar barcode[LB_BARCODE_LEN];  /* barcode label info                 */
	int32_t media_devtype;   /* the PVR Device type.Family of media       */
	int32_t spec_media_type; /* MT_XXX value.0xFFFFFFFF if unknown        */
	int32_t element_devtype; /* element domain. Specifies drive type      */
	int32_t element_drv_type;/* what kind of drive installed (mixed med.) */
	uchar worm;              /* True if media is WORM                     */
	uchar reserverd[15];
} lb_enh_drive_state_t;

typedef struct {
	int32_t compcode;       /* O - completion code                        */
	int32_t drives;         /* I - number of drives                       */
	int32_t slots;          /* I - number of storage slots                */
	int32_t eeports;        /* I - number of entry exit ports             */
	int32_t transp_elems;   /* I - number of transport elems              */
	lb_enh_drive_state_t* enh_drive_state;  /* O - user buffer            */
	lb_enh_storage_element_state_t* enh_storage_element_state; /* O - usr */
	lb_enh_import_export_state_t* enh_import_export_state;     /* O - usr */
	lb_medium_changer_state_t* medium_changer_state;           /* O - usr */
} lb_enh_inventory_t;


/*************************************************************************
 * Definitions for the "medium_type" field in the "dd_get_medium_info"
 * structure.
 ************************************************************************/

#define  DD_UNKNOWN_MEDIA_TYPE    -1
#define  DD_NO_CARTRIDGE_LOADED    0

#define  MT_ULTRIUM    387
#define  MT_ULTRIUM2   394
#define  MT_3592_RW    411
#define  MT_3592_WORM  412
#define  MT_ULTRIUM3   417
#define  MT_3592_RW2   419
#define  MT_3592_WORM2 420
#define  MT_ULTRIUM4   427
#define  MT_ULTRIUM5   430
#define  MT_3592_RW3   431
#define  MT_3592_WORM3 432
#define  MT_ULTRIUM6   436
#define  MT_3592_RW4   437
#define  MT_3592_WORM4 438
#define  MT_ULTRIUM7   439
#define  MT_ULTRIUM8   441
#define  MT_ULTRIUM8M  442
#define  MT_3592_RW5   443
#define  MT_3592_WORM5 444
#define  MT_ULTRIUM9   445

/*
 * LBIOC_XDRIVE_INVENTORY definitions
 */
typedef struct {
	lb_drive_state_t lb_drive_state;
	uchar drive_serial[DD_MAX_DEVICE_SERIAL];  /* drive serial from RES */
	u_int32_t drive_serial_valid;
	uchar reserved[64];
} lb_Xdrive_state_t;

typedef struct {
	int32_t compcode;       /* O - completion code        */
	ushort drive_element;   /* I - drive element addr     */
	int32_t drives;         /* I - number of drives       */
	lb_Xdrive_state_t *Xdrive_state; /* O - Pointer to user buffer */
	uchar reserved[32];
} lb_Xdrive_inventory_t;


/*
 *  LBIOC_LIBRARY_INVENTORY_RANGE
 */
typedef struct {
	int32_t compcode;       /* O - completion code                        */
	int32_t typecode;       /* I - element type                           */
	int32_t start_elem_addr;/* I - starting element address               */
	int32_t total_elem_num; /* I - total element number                   */
	void* state_p;          /* O - pointer to user buffer the state above */
	uchar reserved[44];
} lb_inventory_range_t;

/*
 * LBIOC_MOVE_MEDIUM definitions
 */
typedef struct {
	int32_t compcode;          /* O - request completion code             */
	ushort transport_element;  /* I - transport element address           */
	ushort source_address;     /* I - where we are moving from            */
	ushort destination_address;/* I - where we are moving to              */
	int32_t invert;            /* I - if True, invert before moving       */
} lb_move_medium_t;

/*
 * LBIOC_LIBRARY_AUDIT definitions
 */
typedef struct {
	int32_t compcode;       /* O - request completion code            */
	int32_t barcode;        /* I - scan barcodes?                     */
} lb_audit_t;

/*
 * LBIOC_EXTEND_RETRACT_IO_PT definitions
 */
typedef struct {
	int32_t compcode;       /* O - completion code                        */
	int32_t option;         /* I - extend the import/export mechanism     */
	ushort eeport_addr;     /* I - Address of the entry exit port         */
	ushort transp_elem_addr;/* I - Address of the transport element       */
} lb_extend_retract_t;

/*
 * LBIOC_LIBRARY_INFO definitions
 */
typedef struct {
	uint exchange_medium:1, ier:1, reserved:30;
} lb_capability_t;

typedef struct {
	int32_t compcode;       /* O - completion code           */
	int32_t support_module; /* O - device support module     */
	int32_t robot_can_eject;/* O - robot can eject cartridge */
	int32_t inv_unreliable; /* O - inventory unreliable?     */
	int32_t skip_all_audits;/* O - high performance inven?   */
	int32_t must_position_to_elem; /* O - position required on move */
	int32_t moves_slot_to_slot;    /* O - supports slot-to-slot     */
	int32_t uses_magazine;         /* O - magazine checkin/out      */
	int32_t robot_can_initiate;     /* O - robot acts as initiator   */
	int32_t robot_can_invert;       /* O - robot can invert cartridge */
	int32_t stackable_magazine;     /* 0 - cartridges stackable?     */
	lb_capability_t lb_capable_flag;
	int32_t mixed_devtype_capable;  /* O - can handle multiple devtypes */
	int32_t reserved[7];            /* O - always 0                     */
} lb_library_info_t;

/*
 * LBIOC_POSITION definitions
 */
typedef struct {
	int32_t compcode;               /* 0 - request completion code      */
	ushort transport_element;       /* I - tansport element address     */
	ushort destination_address;     /* I - where to position to         */
	int32_t invert;                 /* I - invert                       */
	uchar reserved[32];             /* reserved                         */
} lb_position_t;

/*
 * LBIOC_EXCHANGE_MEDIUM definitions
 */
typedef struct {
	int32_t compcode;                 /* O - request completion code      */
	ushort transport_element;         /* I - transport element address    */
	ushort source_address;            /* I - where we are moving from     */
	ushort first_destination_address; /* I - first destination            */
	ushort second_destination_address;/* I - second destination           */
	int32_t invert;                   /* I - bit flag for which to invert */
} lb_exchange_medium_t;

/*
 * DDIOC_INQUIRY definitions
 * DDIOC_XINQUIRY definitions
 */
typedef struct {                /* mapping for standard 56 bytes of INQUIRY   */
	uchar device_type;      /* bits 7-5, peripheral qualifier,            */
	uchar device_type_modifier;    /* bit 7 = 1 means media is removable  */
	uchar flags;                   /* bits 7 and 6 - ISO Version          */
	uchar flags2;
	uchar additional_length;       /* Additional bytes of INQUIRY data.   */
	uchar reserved;
	uchar reserved2;
	uchar flags3;
	char vendor_id[DD_MAX_VENDOR_ID]; /* ASCII of vendor ID               */
	char product_id[DD_MAX_PRODUCT_ID]; /* ASCII of product name          */
	char product_revision_level[DD_MAX_REV_LEVEL];
	uchar vendor_spec1[20];                 /* Vendor specific data       */
} dd_inq_data_t;

typedef struct {                /* mapping for device serial INQUIRY          */
	uchar device_type;      /* bits 4-0, peripheral device type           */
	uchar page_code;        /* code page 80 is for device serial          */
	uchar reserved;
	uchar length;
	char inq_dev_serial[DD_MAX_DEVICE_SERIAL]; /* ASCII representation    */
	char reserved2[56 - 4 - DD_MAX_DEVICE_SERIAL]; /* device serial number*/
} dd_inq_serial_t;

typedef struct {                /* mapping for Inquiry page 83                */
	uchar device_type;      /* bits 4-0, peripheral device type           */
	uchar page_code;        /* code page 83 is for wwname                 */
	uchar reserved;
	uchar length;
	char ww_name_buf[56 - 4]; /* strings that might contain the ww_name   */
} dd_inq_wwname_t;

typedef struct {                /* This entire structure is OUTPUT            */
	int32_t compcode;       /* O - request completion code                */
	dd_inq_data_t data;     /* O - standard SCSI inquiry data             */
	char driver_ident[32];  /* O - identify device driver                 */
} dd_old_inquiry_t;

typedef struct {
	uint ww_name_valid:1, dd_device_serial_valid:1, reserved:6;
} dd_valid_flag_t;

typedef struct {                /* Output: All parameters are output */
	int32_t compcode;       /* O - request completion code       */
	dd_inq_data_t data;     /* O - standard SCSI inquiry data    */
	char driver_ident[DD_DRIVER_LEVEL_LENGTH];
	char reserved[32 - DD_DRIVER_LEVEL_LENGTH];
	uchar ww_name[DD_MAX_DEVICE_WWNAME];
	uchar dd_device_serial[DD_MAX_DEVICE_SERIAL];
	dd_valid_flag_t dd_valid_flag;
	char reserved2[252 - 4 - sizeof(dd_inq_data_t)
		- 32 - DD_MAX_DEVICE_WWNAME - DD_MAX_DEVICE_SERIAL
		- sizeof(dd_valid_flag_t)];
} dd_inquiry_t;

/*
* Persistent Reserve declarations and structures
*/

int lin_tape_tsm_query_pr_key(struct tape_descriptor * drv, uint command,
    ulong arg, unchar pseudo);
int lin_tape_tsm_set_reserve_key(struct tape_descriptor* drv, uint command,
	ulong arg);
int lin_tape_tsm_clear_reserve_key(uint command, ulong arg);
int lin_tape_tsm_preempt_abort(uint command, ulong arg);

#define ADSM_DD_MAX_DEV_NAME  64 // MPD SpecialFileNamelength at lintape is 16

typedef struct {
    uchar device_serial[DD_MAX_DEVICE_SERIAL];        /* I - target SN from TSM */
    uchar dev_special_filename[ADSM_DD_MAX_DEV_NAME]; /* I - target device special file name */
    uchar target_pr_key[PRKEY_SIZE];                  /* O - key from target dvc */
    int32_t compcode;                                 /* O - completion code */
} dd_query_prkey_t;

typedef struct {
	uchar tsm_pr_key[PRKEY_SIZE];              /* I - key to set */
	int32_t compcode;                          /* O - completion code */
} dd_set_reserve_key_t;

typedef struct {
	uchar device_serial[DD_MAX_DEVICE_SERIAL]; /* I - target SN from TSM */
	uchar dev_special_filename[ADSM_DD_MAX_DEV_NAME];  /* I - target device special file name */
	int32_t compcode;                          /* O - completion code */
} dd_clear_reserve_t;

typedef struct {
	uchar device_serial[DD_MAX_DEVICE_SERIAL]; /* I - target SN from TSM */
	uchar tsm_pr_key[PRKEY_SIZE];              /* I - key to set */
	uchar target_pr_key[PRKEY_SIZE];           /* I - key from target dvc */
	uchar dev_special_filename[ADSM_DD_MAX_DEV_NAME];  /* I - target device special file name */
	int32_t compcode;                          /* O - completion code */
} dd_preempt_abort_t;

/*
 * DDIOC_RETURN_ERR definitions
 */
typedef struct {
	int32_t last_status;            /* O - request errno                  */
	erp_compcode_t last_compcode;   /* O - request completion code        */
	int32_t information;            /* O - not supported                  */
	int32_t information_valid;      /* O - not supported                  */
	u_int32_t number_sense;         /* O - number of sense bytes returned */
	uchar sense_bytes[DD_SCSI_MAX_SENSE];   /* O - sense bytes            */
} dd_return_error_state_t;

/*
 * DDIOC_LOG_SENSE definitions
 */
/* page_control definitions */
#define DD_PAGE_CONTROL_CUMULATIVE            0x40
#define DD_PAGE_CONTROL_THRESHOLD             0x00
#define DD_PAGE_CONTROL_DEFAULT_THRESHOLD     0x80
#define DD_PAGE_CONTROL_DEFAULT_CUMULATIVE    0xC0

/* page_code definitions */
#define DD_PAGE_CODE_SUPPORTED_PAGES          0x00
#define DD_PAGE_CODE_WRITE_ERROR_COUNTERS     0x02
#define DD_PAGE_CODE_READ_ERROR_COUNTERS      0x03
#define DD_PAGE_CODE_SIM_MIM                  0x31
#define DD_PAGE_CODE_TAPE_ALERT               0x2E

/*
 * log sense structure
 *
 * Structure definition for the log sense ioctl command.
 * This ioctl is used to return a log sense page from a device.
 *
 */
typedef struct {
	int32_t compcode;       /* O - request completion code */
	u_int32_t page_control; /* I - type of data to return  */
	u_int32_t page_code;    /* I - page to return          */
	unsigned char data[DD_MAX_LOG_SENSE]; /* O - returned page */
} dd_log_sense_page_t;

/* convenience definition of a log page header */
typedef struct {
	unsigned char page_code;
	unsigned char reserved;
	unsigned char len[2];
	unsigned char page[1];
} dd_log_page_header_t;

/* convenience definition of a log page parameter */
typedef struct {
	unsigned char code[2];
	unsigned char flag;
	unsigned char len;
	unsigned char data[16];
} dd_log_page_parameter_t;

/* endian independent retrieve page length */
/* tape alert entry structure              */
typedef struct {
	unsigned char alert_code[2];
	unsigned char flag;
	unsigned char len;
	unsigned char alert_value;
} dd_log_tapealert_entry_t;

/* tape alert page  structure */
typedef struct {
	unsigned char pagecode;
	unsigned char flag;
	unsigned char pagelen[2];
	dd_log_tapealert_entry_t tapealert[DD_MAX_TAPEALERT];
} dd_log_tapealert_page_t;

#define DD_LOG_PAGE_LEN(x)    (int32_t)  x[1] + (int32_t) (x[0] << 8)

/* lin_tape extra ioctl functions for medium chagners */
ssize_t changer_ddioc_inquiry(struct changer_descriptor **, void *, char);
ssize_t changer_ddioc_ret_err(struct changer_descriptor **, void *, char);
ssize_t changer_lbioc_lib_info(struct changer_descriptor **, void *, char);
ssize_t changer_lbioc_ret_elem_cnt(struct changer_descriptor **, void *, char);
ssize_t changer_lbioc_move_medium(struct changer_descriptor **, void *, char);
ssize_t changer_lbioc_lib_inventory(struct changer_descriptor **, void *, char);
ssize_t changer_lbioc_position(struct changer_descriptor **chgr, void *buffer,
	char unused);
ssize_t changer_lbioc_audit(struct changer_descriptor **, void *, char);
ssize_t changer_lbioc_ext_ret_ioport(struct changer_descriptor **, void *,
	char);
ssize_t changer_ddioc_xinquiry(struct changer_descriptor **, void *, char);
ssize_t changer_lbioc_xdrive_inventory(struct changer_descriptor **, void *,
	char);
int xinv_res_and_devid(struct changer_descriptor** chgp, ushort* num_drives,
	ushort start, ushort req_drives, unchar** devid_buf,
	ulong* devid_buf_size, ulong* devid_order, unchar** inv_buf,
	ulong* inv_buf_size, ulong* inv_order, unchar* vol_tag);
int xinv_gather_and_copy(struct changer_descriptor* chg,
	struct tape_elem_status* elem_info, struct tape_read_devid* read_devid,
	lb_Xdrive_state_t* xdrive_state, ushort start, int k,
	ushort req_drives, unchar bar_code, char copy_method, int* done);
ssize_t changer_lbioc_logsensepage(struct changer_descriptor **, void *, char);
ssize_t changer_lbioc_lib_enh_inventory(struct changer_descriptor **,
	void *, char);

/* support functions */
int changer_get_compcode(int, struct changer_descriptor *);
int changer_get_lib_inventory(struct changer_descriptor **, void *, char);
int init_inventory(struct changer_descriptor* chg,
	struct elem_assignment** elem_asn,
	ushort* req_drives, ushort* req_ieports, ushort* req_slots,
	ushort* req_robots,
	lb_drive_state_t** drive_state,
	lb_storage_element_state_t** slot_state,
	lb_import_export_state_t** ieport_state,
	lb_medium_changer_state_t** robot_state,
	void* buffer);
int changer_get_lib_enh_inventory(struct changer_descriptor **, void *, char);
int inv_res(struct changer_descriptor** chg, void* elem_state, int report_type,
	ushort req_elems, char copy_method, struct elem_assignment* elem_asn,
	boolean enh);
int set_drive_state(struct changer_descriptor* chg, char* data_buf,
	ushort req_drives, unchar bar_code, void* driver_buffer,
	void** user_buffer, char copy_method, boolean enh);
int set_ieport_state(struct changer_descriptor* chg, char* data_buf,
	ushort req_ieports, unchar bar_code, void* driver_buffer,
	void** user_buffer, char copy_method, boolean enh);
int set_storage_state(struct changer_descriptor* chg, char* data_buf,
	ushort count, unchar bar_code, void* driver_buffer,
	void** user_buffer, char copy_method, boolean enh);
int set_robot_state(struct changer_descriptor* chg, char* data_buf,
	ushort req_robots, unchar bar_code, void* driver_buffer,
	void** user_buffer, char copy_method, boolean enh);
void set_barcode(struct changer_descriptor* chg, char* barcode,
	uchar* barcode_flag, int32_t* barcode_len,
	struct tape_elem_status* elem_info);
void set_media_type(int32_t* media_devtype, int32_t* element_devtype,
	int32_t* spec_media_type, uchar* worm, uchar full, uchar barcode_flag,
	int32_t barcode_len, char* barcode, struct tape_elem_status* elem_info);
int changer_get_abnormal_state(ushort);
int lin_tape_get_drv_by_sn(struct tape_descriptor** drv, char* sn);

#endif
