/*******************************************************************************
 *   lin_tape_scsi_config.h
 *
 *   (C) Copyright International Business Machines  Corp., 2007-2020
 *   This file is part of the IBM Linux Enhanced SCSI Tape and
 *   Medium Changer Device Driver.
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ******************************************************************************/

#ifndef _LIN_TAPE_SCSI_CONFIG_H_
#define _LIN_TAPE_SCSI_CONFIG_H_

#include <linux/kernel.h>	/* general kernel support                     */
#include <linux/module.h>	/* general module support                     */

#include <linux/init.h>		/* module_init; module_exit                   */
#include <linux/moduleparam.h>	/* driver parameters                          */
#include <linux/proc_fs.h>	/* proc system                                */
#include <linux/device.h>	/* struct class, class_simple, device_driver  */
#include <linux/sysfs.h>	/* sysfs_create/remove_link, etc.             */
#include <linux/kdev_t.h>	/* MKDEV/MAJOR/MINOR macros                   */
#include <linux/fs.h>		/* file operations                            */
#include <linux/kobject.h>	/* struct kobject                             */
#include <linux/cdev.h>		/* struct cdev, cdev_alloc, ..                */
#include <linux/blkdev.h>       /* struct request, request_queque             */
#include <linux/version.h>      /* kernel version number                      */
#ifndef UTS_RELEASE
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,0,0)
#include <linux/utsrelease.h>
#else
#include <generated/utsrelease.h>
#endif
#endif
#include <linux/wait.h>         /* wait queue                                 */
#include <linux/delay.h>        /* delay on configuration                     */

#include <scsi/scsi_device.h>	/* struct scsi_device;
				 * to_scsi_device                             */
#include <scsi/scsi.h>		/* scsi command definition, etc.              */

#include <scsi/scsi_driver.h>	/* struct scsi_driver; to_scsi_driver;        *
				 * scsi_register/unregister_driver,           *
				 * scsi_register/unregister_interface         */
#include <scsi/scsi_host.h>	/* struct scsi_host_template;                 *
				 * struct Scsi_Host                           */

/* module information */
MODULE_AUTHOR("IBM Corporation");
MODULE_DESCRIPTION
    ("IBM Linux SCSI Tape Device Driver for IBM and other vendor Tape Devices");
MODULE_LICENSE("GPL");
MODULE_VERSION("3.0.56");
/* driver parameters */
int support_dio = 1;
module_param(support_dio, int, 0);
MODULE_PARM_DESC(support_dio, " try direct I/O if possible. default is "\
	"1--yes. 0--do not try.");

int alternate_pathing = 0;
module_param(alternate_pathing, int, S_IRUGO);
MODULE_PARM_DESC(alternate_pathing, " try to enable path failover on all devices. default is 0--off.  1--try to enable.");

int persistent_n_device = 0;
module_param(persistent_n_device, int, S_IRUGO);
MODULE_PARM_DESC(persistent_n_device, " special configuration to allow persistent "\
    "naming for no rewind on close tape drives.  Special rules must be used. "\
    "default is 0--OFF.  1--ON");

char *dpf_keys = NULL;
module_param(dpf_keys, charp, 0);
MODULE_PARM_DESC(dpf_keys, " data path failover license key for LTO2/3 "\
	"drives. default is null.");

int lin_tape_debug = 0;
module_param(lin_tape_debug, int, S_IRUGO);
MODULE_PARM_DESC(lin_tape_debug, " driver debug mode. default is 0--off. "\
	"1--on");

int auto_logging = 0;
module_param(auto_logging, int, S_IRUGO);
MODULE_PARM_DESC(auto_logging, " automatically write volume & simmim logs. "\
	"default is 0--off. 1--on");

int lin_tape_clean_ghosts = 0;
module_param(lin_tape_clean_ghosts, int, S_IRUGO);
MODULE_PARM_DESC(lin_tape_clean_ghosts, " clean ghosts on rescan. default is 0--off. "\
	"1--on");

int lin_tape_status = 0;
module_param(lin_tape_status, int, S_IRUGO);
MODULE_PARM_DESC(lin_tape_status, " driver debug mode. for L3 only");

int changer_open_reserve = 0;
module_param(changer_open_reserve, int, S_IRUGO);
MODULE_PARM_DESC(changer_open_reserve, " reserve all medium changers on open."\
	"  default is 0--off.  1--on");

int lto_barcode = 8;
module_param(lto_barcode, int, S_IRUGO);
MODULE_PARM_DESC(lto_barcode, " returned cartridge bar code length for LTO. "\
	"default is 8. can be changed to 6");

int ibm3592_barcode = 6;
module_param(ibm3592_barcode, int, S_IRUGO);
MODULE_PARM_DESC(ibm3592_barcode, " returned cartridge bar code length for "\
	"3592. default is 6. can be changed to 8");

int petro = 0;
module_param(petro, int, 0);
MODULE_PARM_DESC(petro, " special configuration for seismic data. default is "\
	"0--OFF.  1--ON");

int wait_on_config = 0;
module_param(wait_on_config, int, S_IRUGO);
MODULE_PARM_DESC(wait_on_config, " milliseconds to wait during device "\
	"configuration. default is 0. suggest to set it to 1000 or higher if "\
	"you have >100 devices attached to one host.");

int default_sys_encryption_proxy = 1;
module_param(default_sys_encryption_proxy, int, S_IRUGO);
MODULE_PARM_DESC(default_sys_encryption_proxy, " default setting to allow "\
	"driver to talk to encryption key manager. default is on.  suggest "\
	"always keep it on.");

int default_sys_encryption_write = 2;
module_param(default_sys_encryption_write, int, S_IRUGO);
MODULE_PARM_DESC(default_sys_encryption_write, " default encryption policy "\
	"setting for all drives. default is 2--custom.  1--enable encryption "\
	"when system-managed encryption method is configured on all drives, "\
	"0--disable encryption when System-managed encryption method is "\
	"configured on all drives");

#define RESERVE_TYPE_LENGTH (16)
char tape_reserve_type[RESERVE_TYPE_LENGTH] = "reserve_6";
module_param_string(tape_reserve_type, tape_reserve_type, RESERVE_TYPE_LENGTH,
	S_IRUGO);
MODULE_PARM_DESC(tape_reserve_type, " default reserve type when data path "\
"failover is not used. Default is \"reserve_6\". Can also be \"persistent\"");

int disable_density_on_open = 0;
module_param(disable_density_on_open, int, S_IRUGO);
MODULE_PARM_DESC(disable_density_on_open, " determines whether setting density"\
	" on open will be blocked.  default is 0--do not block (IE set "\
	"density) 1--block (IE do not set density)");

int dynamic_attributes = 1;
module_param(dynamic_attributes, int, S_IRUGO);
MODULE_PARM_DESC(dynamic_attributes, " attempt to set dynamic attributes. "\
	"default is 1--on. 0--off");

int busy_retry = 0;
module_param(busy_retry, int, S_IRUGO);
MODULE_PARM_DESC(busy_retry, " retry attempts on device busy status. "\
	"default is 0--off. Up to 480 retry times to be specified");

int disable_auto_drive_dump = 0;
module_param(disable_auto_drive_dump, int, S_IRUGO);
MODULE_PARM_DESC(disable_auto_drive_dump, " Disable auto drive dump. "\
	"default is 0--off. See lin_taped.conf for special/general log configuration parameters");

int lin_tape_as_sfmp = 0;
module_param(lin_tape_as_sfmp, int, S_IRUGO);
MODULE_PARM_DESC(lin_tape_as_sfmp, " sfmp mode. default is 0--off. "\
	"1--on. If failvoer enabled lin_tape_as_sfmp enables failover through st/sg interfaces");

int dualaccessor_cpf = 0;
module_param(dualaccessor_cpf, int, S_IRUGO);
MODULE_PARM_DESC(dualaccessor_cpf, " CPF mode for dual accessor. default is 0--off. "\
	    "1--on");

/* global external functions */
static int lin_tape_init(void);
int initialize_lin_tape_class(void);
int alloc_and_register(dev_t** first_dev, int* chrdev_regd, int* driver_regd,
	int* interface_regd);
void create_driver_attributes(void);
void lin_tape_undo_init(int interface_regd, int chrdev_regd, int driver_regd,
	int lin_tape_class_allocd, dev_t* first_dev);

static void lin_tape_exit(void);

int lin_tape_sdev_get(struct scsi_device* sdev, int* ct);
void lin_tape_sdev_put(struct scsi_device* sdev, int* ct);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
static int lin_tape_add_class_device(struct class_device *);
static void lin_tape_remove_class_device(struct class_device *);
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
static int lin_tape_add_class_device(struct class_device *,
	struct class_interface*);
static void lin_tape_remove_class_device(struct class_device *,
	struct class_interface*);
#else
static int lin_tape_add_class_device(struct device *, struct class_interface*);
static void lin_tape_remove_class_device(struct device *,
	struct class_interface*);
#endif
static void lin_tape_rescan(struct device *);

static int tape_read_proc
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
    (char* buffer, char** start, off_t offset, int length, int* eof, void* data);
#else
    (struct seq_file *, void *);
static int tape_open_proc(struct inode *inode, struct file *file)
{
	return single_open(file, tape_read_proc, NULL);
}
#if LINUX_VERSION_CODE < KERNEL_VERSION(5,10,0)
static struct file_operations tape_proc_ops = {
    .owner   = THIS_MODULE,
    .open    = tape_open_proc,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = single_release,
};
#else
static struct proc_ops tape_proc_ops = {
    .proc_open    = tape_open_proc,
    .proc_read    = seq_read,
    .proc_lseek  = seq_lseek,
    .proc_release = single_release,
};
#endif
#endif

int rd_pc_initial
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
    (char* , int* , int , char** , boolean);
#else
    (struct seq_file *, boolean);
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
int rd_pc_start_dev(int* start_dev, int* eof, off_t offset, char* buffer,
		int* buf_offset, boolean chg);
#endif

char* scsi_addr_str(struct tape_descriptor* drv, char* buffer, boolean cgr);

static int changer_read_proc
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
    (char *, char **, off_t, int, int *, void *);
#else
    (struct seq_file *, void*);
static int changer_open_proc(struct inode *inode, struct file *file)
{
	return single_open(file, changer_read_proc, NULL);
}
#if LINUX_VERSION_CODE < KERNEL_VERSION(5,10,0)
static struct file_operations changer_proc_ops = {
    .owner   = THIS_MODULE,
    .open    = changer_open_proc,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = single_release,
};
#else
static struct proc_ops changer_proc_ops = {
    .proc_open    = changer_open_proc,
    .proc_read    = seq_read,
    .proc_lseek  = seq_lseek,
    .proc_release = single_release,
};
#endif
#endif

static int status_read_proc
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
    (char *, char **, off_t, int, int *, void *);
#else
    (struct seq_file *, void *);
static int status_open_proc(struct inode *inode, struct file *file)
{
	return single_open(file, status_read_proc, NULL);
}
#if LINUX_VERSION_CODE < KERNEL_VERSION(5,10,0)
static struct file_operations status_proc_ops = {
    .owner   = THIS_MODULE,
    .open    = status_open_proc,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = single_release,
};
#else
static struct proc_ops status_proc_ops = {
    .proc_open    = status_open_proc,
    .proc_read    = seq_read,
    .proc_lseek  = seq_lseek,
    .proc_release = single_release,
};
#endif
#endif

int rdstatus_pc_initial
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
    (char* , int* , int , char** , boolean);
#else
    (struct seq_file *, boolean);
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
int rdstatus_pc_start_dev(int* start_dev, int* eof, off_t offset, char* buffer,
		int* buf_offset, boolean chg);
#endif

/* global variables */
static unchar lin_tape_cleanup_module = 0;
static struct cdev* lin_tape_daemon_dev = NULL;
static struct cdev* lin_tape_tsm_dev = NULL;
char lin_tape_trace_type = 0;
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,0,0)
rwlock_t lin_tape_trace_lock = RW_LOCK_UNLOCKED;
#else
DEFINE_RWLOCK(lin_tape_trace_lock);
#endif
unchar *lin_tape_log = NULL;
uint lin_tape_log_len = 0;
static struct class *lin_tape_class = NULL;
#define LIN_TAPE_CLASS_DEV_ATTRS  3
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,14,0)
static const struct attribute_group *lin_tape_dev_groups[];
static const struct attribute_group *lin_tape_drv_groups[];
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,0,0)
rwlock_t tape_des_lock = RW_LOCK_UNLOCKED;	/* rwlock for drive table     */
rwlock_t changer_des_lock = RW_LOCK_UNLOCKED;	/* rwlock for changer table   */
rwlock_t lin_tape_lock = RW_LOCK_UNLOCKED;	/* rwlock for driver globals  */
#else
DEFINE_RWLOCK(tape_des_lock);	/* rwlock for drive table     */
DEFINE_RWLOCK(changer_des_lock);	/* rwlock for changer table   */
DEFINE_RWLOCK(lin_tape_lock);	/* rwlock for driver globals  */
#endif

struct descriptor_tape **tape_devs = NULL;
struct descriptor_changer **changer_devs = NULL;
struct tape_descriptor* tsm_dev = NULL;

atomic_t tsm_tp_open = {0};
static unsigned int lin_tape_major;

//#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
static struct proc_dir_entry *tape_proc_entry = NULL;
static struct proc_dir_entry *changer_proc_entry = NULL;
static struct proc_dir_entry *status_proc_entry = NULL;
//#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,0,0)
rwlock_t hba_lock = RW_LOCK_UNLOCKED;
#else
DEFINE_RWLOCK(hba_lock);
#endif
struct adapter_descriptor *hba_tab = NULL;

/* class interface template */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
static struct class_interface lin_tape_class_interface = {
	.add = lin_tape_add_class_device,
	.remove = lin_tape_remove_class_device,
};
#else
static struct class_interface lin_tape_class_interface = {
	.add_dev = lin_tape_add_class_device,
	.remove_dev = lin_tape_remove_class_device,
};
#endif


/* scsi driver template */
static int lin_tape_probe(struct device *dev);
static int lin_tape_remove(struct device *dev);
static struct scsi_driver lin_tape_driver = {
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,19,0)
	.owner = THIS_MODULE,
#endif
	.gendrv = {
		   .name = "lin_tape",
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,4,21)
		.owner			= THIS_MODULE,
#endif
		   .probe = lin_tape_probe,
		   .remove = lin_tape_remove,
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,14,0)
		   .groups = lin_tape_drv_groups,
#endif
		   },
	.rescan = lin_tape_rescan,
};

/* to be used in configuring and unconfiguring the different kinds of drives */
#define CFG_SYSFS_DEV   (1)
#define CFG_SYSFS_LNK   (1 << 1)
#define CFG_SYSFS_SN    (1 << 2)
#define CFG_SYSFS_WWNN  (1 << 3)
#define CFG_SYSFS_WWPN  (1 << 4)
#define CFG_SYSFS_PATH  (1 << 5)
#define CFG_SYSFS_EPXY  (1 << 6)
#define CFG_SYSFS_EWRT  (1 << 7)

#define CFG_PRIM_DVC    (0xFF)
#define CFG_ALT_DVC     (CFG_SYSFS_DEV | CFG_SYSFS_EPXY | CFG_SYSFS_EWRT)
#define CFG_ALT_DVC_N   (CFG_SYSFS_DEV | CFG_SYSFS_EPXY | CFG_SYSFS_EWRT \
			| CFG_SYSFS_SN | CFG_SYSFS_WWNN | CFG_SYSFS_WWPN)
#define CFG_CHG_DVC     (CFG_SYSFS_DEV | CFG_SYSFS_LNK | CFG_SYSFS_SN \
			| CFG_SYSFS_PATH)

#define TOT_RETRIES (3)

/* Busy flags */
#define F_ADD_REMOVE (1)
#define F_SEND      (1 << 1)
#define F_OPEN      (1 << 2)
#define F_CLOSE     (1 << 3)
#define F_SCAN      (1 << 4)
#define F_READ      (1 << 5)
#define F_WRITE     (1 << 6)
#define F_IOCTL     (1 << 7)

/* file operation functions */
static ssize_t lin_tape_read(struct file *, char *, size_t, loff_t *);
static ssize_t lin_tape_write(struct file *, const char *, size_t, loff_t *);
static int lin_tape_open(struct inode *, struct file *);
int lin_tape_open_pseudo(void);
int lin_tape_open_tsm(void);
int lin_tape_open_drive(int minor, unchar ttype, struct file* filp);
int lin_tape_get_drive_des(struct tape_descriptor** drv, int minor,
	unchar ttype, unchar* reserve_on_open);
int lin_tape_get_changer_des(struct changer_descriptor** chg, int minor,
	unchar ttype);
int lin_tape_find_drv_path(struct tape_descriptor** drv);
int lin_tape_drv_open_check(struct tape_descriptor* drv, unchar ttype);
int lin_tape_open_changer(int minor, unchar ttype, struct file* filp);
static int lin_tape_close(struct inode *, struct file *);
int lin_tape_close_drive(struct file* filp, int minor);
int lin_tape_close_changer(struct file* filp, int minor);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
static int lin_tape_ioctl(struct inode *, struct file *, uint, ulong);
#else
static long lin_tape_ioctl(struct file *, uint, ulong);
#endif
extern int drv_gen(struct tape_descriptor*);
extern void printk_hex(char*, int, int);
int lin_tape_write_dynamic_attrs(struct tape_descriptor** drvp, boolean clear);
int get_dpf_path(struct tape_descriptor* drv);
int lin_tape_ioctl_drive(int minor, struct file* filp, uint command, ulong arg);
int lin_tape_ioctl_changer(int minor, struct file* filp, uint command,
	ulong arg);

static struct file_operations lin_tape_ops = {
	.owner = THIS_MODULE,
	.read = lin_tape_read,
	.write = lin_tape_write,
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
	.ioctl = lin_tape_ioctl,
#else
	.unlocked_ioctl = lin_tape_ioctl,
#endif
	.open = lin_tape_open,
	.release = lin_tape_close,
};

/* internal functions */
void lin_tape_create_proc(void);
void lin_tape_remove_proc(void);
int lin_tape_supported_device(struct scsi_device* device);
int lin_tape_config_device(struct device*);
void lin_tape_bind_device(struct device* dev);
int lin_tape_config_drive(int tape_id, struct scsi_device* sdev);
int lin_tape_config_changer(int tape_id, struct scsi_device* device);
int allocate_tdesc_array(void);
int allocate_cdesc_array(void);
int alloc_and_init_tdesc(struct scsi_device* sdev, int tape_id, int* i,boolean rescan);
int alloc_and_init_cdesc(struct scsi_device* sdev, int tape_id, int* i,boolean rescan);
void cleanup_and_free_tdesc(int idx);
void cleanup_and_free_cdesc(int idx);
void lin_tape_unconfig_device(struct device*);
void unconfig_drive_paths(struct device* dev);
void unconfig_changer_paths(struct device* dev);
int lin_tape_unconfig_drive(struct device* dvc, struct tape_descriptor *drv);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
void lin_tape_uncfg_file_tp_rm(struct scsi_device* sdev,
	struct class_device** dvc, int idx, ulong flags);
#else
void lin_tape_uncfg_file_tp_rm(struct device** dvc, int idx, ulong mask);
#endif

int lin_tape_unconfig_changer(struct device* dvc, struct scsi_device* sdev,
	int idx);
void lin_tape_uncfg_file_cg_rm(int idx, ulong flags);
void lin_tape_drive_cleanup(struct tape_descriptor *drv);
void lin_tape_changer_cleanup(int);
int lin_tape_create_tsm_pseudo(struct scsi_device* sdev);
void lin_tape_delete_tsm_pseudo(void);

int lin_tape_init_drive_des(struct tape_descriptor*, struct scsi_device*,
	int *i, boolean rescan);
int lin_tape_create_cdev(struct cdev** cd, int idx, int offset, char* name);


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
int lin_tape_register_class_device(struct class_device** cd,
	struct tape_descriptor* drv, struct scsi_device* sdev, char* name,
	int idx, int offset, ulong* flags);
void lin_tape_undo_class_dvc(struct class_device** cd, struct scsi_device* sdev,
	 int idx, int offs, ulong flags);
#else
int lin_tape_register_class_device(struct device** cd,
	struct tape_descriptor* drv, struct scsi_device* sdev, char* name,
	int idx, int offset, ulong* flags);
void lin_tape_undo_class_dvc(struct device** cd, struct scsi_device* sdev,
	int idx, int offs, ulong flags);
#endif


void lin_tape_set_initial_drv_values(struct tape_descriptor* drv,
	struct scsi_device* sdev);
int check_for_dup_drive(struct tape_descriptor** drvp,
	struct scsi_device* sdev, int* i, boolean rescan);
int lin_tape_init_changer_des(struct changer_descriptor*, struct scsi_device*,
	int* i, boolean rescan);
void lin_tape_set_initial_chg_values(struct changer_descriptor* chg,
	struct scsi_device* sdev);
int check_for_dup_changer(struct changer_descriptor** drv,
	struct scsi_device* sdev, int* i, boolean rescan);
int check_for_dup_changer_by_wwnn(struct changer_descriptor** drv,
	struct scsi_device* sdev, int* i);
void lin_tape_get_inq_p0x83_data(struct tape_descriptor* drv, int changer);
int lin_tape_inq0x83_ser(char* page, struct tape_descriptor* drv, int i,
	int changer);
void lin_tape_inq0x83_wwn(char* page, struct tape_descriptor* drv, int i,
	int changer);

extern int lin_tape_drive_open(struct tape_descriptor**, struct file*, unchar);
extern int lin_tape_changer_open(struct changer_descriptor**, struct file*);
extern int lin_tape_drive_close(struct tape_descriptor**, struct file*, unchar);
extern int lin_tape_changer_close(struct changer_descriptor**, struct file*);
extern int lin_tape_drive_ioctl(struct tape_descriptor**, struct file*, uint,
	ulong);
extern int lin_tape_changer_ioctl(struct changer_descriptor**, struct file*,
	uint, ulong);
extern int lin_tape_daemon_ioctl(uint, ulong);
extern ssize_t lin_tape_drive_read(struct tape_descriptor**, struct file*,
	char*, size_t, loff_t*);
extern ssize_t lin_tape_drive_write(struct tape_descriptor**, struct file*,
	const char*, size_t, loff_t*);
extern int drive_busy(struct tape_descriptor *drv, unchar flag);
extern int changer_busy(struct changer_descriptor *drv, unchar flag);
void lin_tape_rescan_tape(struct scsi_device* sdev, int tape_id);
void lin_tape_rescan_changer(struct scsi_device* sdev, int tape_id);
extern int lin_tape_pass_thru_from_pfoID(struct tape_descriptor **drvp,
				unchar* cdb, unchar cmd_length, unchar* buffer,
				uint *buffer_length, uint data_direction, uint timeout,
				unchar* sense, unchar *sense_length, unchar *sense_avail,
				int *resid, int *errors);
int lin_tape_changer_pass_thru_from_pfoID(struct changer_descriptor **chgp,
				unchar* cdb, unchar cmd_length, unchar* buffer,
				uint *buffer_length, uint data_direction, uint timeout,
				unchar* sense, unchar *sense_length, unchar *sense_avail,
				int *resid, int *errors	);

static struct cdev* lin_tape_special_dev = NULL;
struct tape_descriptor* special_dev = NULL, * special_drv = NULL;
atomic_t special_tp_open = {0};
int lin_tape_open_special(void);
int lin_tape_ioctl_special(struct tape_descriptor **drv, uint command, ulong arg);
int lin_tape_create_special_dev(struct scsi_device* sdev);
void lin_tape_delete_special_dev(void);
void copy_wwn_from_tape_devs(struct changer_descriptor* chg);

#endif
