/*******************************************************************************
 *   lin_tape_scsi_tape.h
 *
 *   (C) Copyright International Business Machines  Corp., 2007-2020
 *   This file is part of the IBM Linux Enhanced SCSI Tape and
 *   Medium Changer Device Driver.
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ******************************************************************************/

#ifndef _LIN_TAPE_SCSI_TAPE_H_
#define _LIN_TAPE_SCSI_TAPE_H_

#include <linux/kernel.h>	/* general kernel support       */
#include <linux/device.h>
#include "lin_tape_tape.h"

extern int lin_tape_debug;
extern int changer_open_reserve;
extern int dualaccessor_cpf;
extern int petro;
extern int support_dio;
extern int disable_auto_dump_drive;
extern int busy_retry;
extern int disable_density_on_open;
extern unchar* tape_log;
extern uint tape_log_len;
extern rwlock_t tape_des_lock;
extern rwlock_t changer_des_lock;
extern struct descriptor_tape **tape_devs;
extern struct descriptor_changer **changer_devs;
extern rwlock_t hba_lock;
extern struct adapter_descriptor *hba_tab;

extern int lin_tape_sdev_get(struct scsi_device* sdev, int* ct);
extern void lin_tape_sdev_put(struct scsi_device* sdev, int* ct);

extern int lin_tape_init_drive_des(struct tape_descriptor*,
	struct scsi_device*, int *i, boolean);
extern int lin_tape_init_changer_des(struct changer_descriptor*,
	struct scsi_device*, int *, boolean rescan);
extern int lin_tape_config_device(struct device*);
extern void lin_tape_unconfig_device(struct device*);

/* internal private funtions */
int lin_tape_write_dynamic_attrs(struct tape_descriptor** drvp, boolean clear);
int lin_tape_dyn_attrs_parm_list(struct tape_descriptor* drv, char* parm_list,
	int len, boolean clear);
int get_dpf_path(struct tape_descriptor* drv);
int drv_gen(struct tape_descriptor* drv);
int lin_tape_drive_open(struct tape_descriptor**, struct file*, unchar);
int set_descriptor_data_on_open(struct tape_descriptor** drvp);
int allocate_tdescriptor_buffers(struct tape_descriptor* drv);
int drv_validate_inquiry(struct tape_descriptor** drvp);
int check_persistent_reservation(struct tape_descriptor** drvp);
int establish_persistent_reservation(struct tape_descriptor** drvp);
int density_and_encryption_on_open(struct tape_descriptor** drvp);
int tape_initial_position(struct tape_descriptor** drvp, struct file* filp,
	unchar ready);
void report_open_drive(struct tape_descriptor* drv, int rc);
void handle_open_failure(struct tape_descriptor** drvp, int rsv);
int lin_tape_changer_open(struct changer_descriptor**, struct file*);
int chg_alloc_cdb_and_sense(struct changer_descriptor* chg);
int chg_validate_vendor_and_serial(struct changer_descriptor** chgp);
int drive_busy(struct tape_descriptor *,unchar flag);
int lin_tape_drive_close(struct tape_descriptor**, struct file*, unchar);
int drive_release_on_close(struct tape_descriptor** drvp, int rc);
void cleanup_buffers_on_close(struct tape_descriptor* drv);
int lin_tape_changer_close(struct changer_descriptor**, struct file*);

int lin_tape_daemon_ioctl(uint, ulong);
int lin_tape_set_trace_type(uint command, ulong arg);
int lin_tape_get_drive_num(ulong arg);
int lin_tape_get_changer_num(ulong arg);
int lin_tape_get_fo_parms(ulong arg);
void lin_tape_set_fo(ulong arg);
void lin_tape_set_fo_drive(ulong arg);
void lin_tape_set_fo_changer(ulong arg);
void lin_tape_set_fo_changer_by_wwnn(ulong arg);
void lin_tape_set_sfmp(ulong arg);
int lin_tape_poll_trace(uint command, ulong arg);
int lin_tape_poll_trace_drive(struct tape_poll* poll, struct tape_trace* trc,
	void** tptr, int* j);
int lin_tape_poll_trace_changer(struct tape_poll* poll, struct tape_trace* trc,
	void** tptr, int* j);
void adjust_queued_dpf(struct tape_descriptor* drv,
	struct tape_full_trace_unit* unit);
void adjust_queued_cpf(struct changer_descriptor* chg,
	struct tape_full_trace_unit* unit);
int lin_tape_retrieve_drive_dump(uint command, ulong arg);
int lin_tape_retrieve_log_data(uint command, ulong arg);
int lin_tape_retrieve_simmim_data(uint command, ulong arg);
int lin_tape_wake_up(ulong arg);
int lin_tape_send_command(ulong arg);
int lin_tape_send_cmd_drive(ulong arg, unchar* buffer, int buf_size,
	struct device_command_t* dev_command);
int lin_tape_send_cmd_changer(ulong arg, unchar* buffer, int buf_size,
	struct device_command_t* dev_command);
int lin_tape_handle_erp(ulong arg);
int lin_tape_handle_erpu(ulong arg);

/* Functions associated with drive IOCTLs */
int lin_tape_drive_ioctl(struct tape_descriptor**, struct file*, uint, ulong);
typedef ssize_t (*drv_method)(struct tape_descriptor**, void*, char);
void log_tape(struct tape_descriptor* drv, char trc_msg);
int drvioc_exe(struct tape_descriptor** drvp, uint command, ulong arg,
	ssize_t buflen, char trc_msg, char cpy_method, boolean trace,
	boolean chk_sz,  boolean fm, boolean to, drv_method func);
ssize_t mttell(struct tape_descriptor** drvp, struct mtop* mt_cmd, ulong arg);
int mtiocget(struct tape_descriptor** drvp, uint command, ulong arg);
int mtiocpos(struct tape_descriptor** drvp, ulong arg);
ssize_t mtdevice(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t stiocdm(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t stioctop(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t stiochgp(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t sioc_inquiry(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t sioc_reqsense(struct tape_descriptor** drvp, void* buf, char unused);
int sioc_reserve(struct tape_descriptor** drvp);
int sioc_release(struct tape_descriptor** drvp);
ssize_t sioc_log_sense_page(struct tape_descriptor** drvp, void* buf,
	char unused);
ssize_t sioc_log_sense10_page(struct tape_descriptor** drvp, void* buf,
	char unused);
ssize_t sioc_enh_log_sense(struct tape_descriptor** drvp, ulong user);
ssize_t sioc_mode_sense(struct tape_descriptor** drvp, void* buf,
	char unused);
ssize_t sioc_mode_sense_page(struct tape_descriptor** drvp, void* buf,
	char unused);
ssize_t sioc_inquiry_page(struct tape_descriptor** drvp, void* buf,
	char unused);
ssize_t stiocsetp(struct tape_descriptor** drvp, void* buf, char unused);
int stiocsetp_check_inval(struct tape_descriptor* drv,
	struct stchgp_s* parm_buf);
int stiocsetp_cap_scal(struct tape_descriptor** drvp,
	struct stchgp_s* parm_buf);
ssize_t stiocqryp(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t stiocqrypos(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t stiocsetpos(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t stiocqrysense(struct tape_descriptor** drvp, void* buf, char unused);
ssize_t stioc_locate(struct tape_descriptor** drvp, void* buf, char loc_16);
int stioc_read_position(struct tape_descriptor** drvp, ulong arg);
ssize_t stioc_report_density_support(struct tape_descriptor** drvp, void* buf,
	char unused);
int get_encryption_state(struct tape_descriptor** drvp, ulong arg);
int set_encryption_state(struct tape_descriptor** drvp, ulong arg);
int set_data_key(struct tape_descriptor** drvp, ulong arg);
int data_encryption_diagnostics(struct tape_descriptor** drvp, ulong arg);
int lin_tape_tsioc_enable_path(struct tape_descriptor** drvp, ulong arg);
int lin_tape_tsioc_disable_path(struct tape_descriptor** drvp, ulong arg);
int lin_tape_tscsi_pass_through(struct tape_descriptor** drvp, uint command,
	ulong arg);
int lin_tape_tsioc_pass_through(struct tape_descriptor** drvp, uint command,
	ulong arg);
int stioc_read_reserve_keys(struct tape_descriptor** drvp, uint command,
	ulong arg);
int stioc_read_reservations(struct tape_descriptor** drvp, uint command,
	ulong arg);
int stioc_register_key(struct tape_descriptor** drvp);
int stioc_remove_registration(struct tape_descriptor** drvp);
int stioc_clear_all_registrations(struct tape_descriptor** drvp);

/* Functions associated with changer IOCTLs */
int lin_tape_changer_ioctl(struct changer_descriptor**, struct file*, uint,
	ulong);
int changer_wait_to_set_busy(struct changer_descriptor** chgp, char *,unchar);
int changer_busy(struct changer_descriptor *,unchar flag);
typedef ssize_t (*chg_method)(struct changer_descriptor**, void*, char);
void log_changer(struct changer_descriptor* chg, char trc_msg, boolean rtrace);
int chgioc_exe(struct changer_descriptor** chgp, uint command, ulong arg,
	ssize_t buflen, char trc_msg, char cpy_method, boolean rtrace,
	boolean fm, boolean to, chg_method);
ssize_t chg_sioc_inquiry(struct changer_descriptor** chgp, void* buf,
	char unused);
ssize_t chg_sioc_inquiry_page(struct changer_descriptor** chgp, void* buf,
	char unused);
int chg_sioc_reqsense(struct changer_descriptor** chgp, uint command,
	ulong arg);
int chg_sioc_reserve_release(struct changer_descriptor** chgp,
	boolean reserve);
ssize_t chg_sioc_log_sense_page(struct changer_descriptor** chgp, void* buf,
	char unused);
ssize_t chg_sioc_log_sense10_page(struct changer_descriptor** chgp, void* buf,
	char unused);
ssize_t chg_sioc_mode_sense_page(struct changer_descriptor** chgp, void* buf,
	char unused);
int lin_tape_csioc_enable_path(struct changer_descriptor* chg, ulong arg);
int lin_tape_csioc_disable_path(struct changer_descriptor* chg, ulong arg);
int smcioc_element_info(struct changer_descriptor** chgp, uint command,
	ulong arg);
ssize_t smcioc_pos_to_elem(struct changer_descriptor** chgp, void* buf,
	char unused);
ssize_t smcioc_init_elem_stat_range(struct changer_descriptor** chgp,
	void* buf, char unused);
ssize_t smcioc_inventory(struct changer_descriptor** chgp, void* buf,
	char cpy_method);
ssize_t smcioc_move_medium(struct changer_descriptor** chgp, void* buf,
	char unused);
ssize_t smcioc_exchange_medium(struct changer_descriptor** chgp, void* buf,
	char unused);
ssize_t smcioc_read_element_devids(struct changer_descriptor** chgp, void* buf,
	char cpy_method);
int smcioc_load_unload_medium(struct changer_descriptor** chgp, uint command,
	ulong arg, boolean load);
ssize_t lin_tape_query_driver_version(struct tape_descriptor** unused1,
	void* buf, char unused2);
int lin_tape_cscsi_pass_through(struct changer_descriptor** chgp,
	uint command, ulong arg);
int lin_tape_cscsi_pass_through(struct changer_descriptor** chgp,
	uint command, ulong arg);
int lin_tape_csioc_pass_through(struct changer_descriptor** chgp, uint command,
	ulong arg);
int lin_tape_enh_lib_inv(struct changer_descriptor** chgp, uint command,
	ulong arg);
int lin_tape_log_sense(struct changer_descriptor** chgp, uint command,
	ulong arg);

ssize_t lin_tape_drive_read(struct tape_descriptor**, struct file*, char*,
	size_t, loff_t*);
int lin_tape_drive_read_check(struct tape_descriptor** drvp);
int prepare_fixed_length_read(struct tape_descriptor* drv, size_t xfer_length,
	size_t* per_xfer_len, size_t* xfer_cnt);
int prepare_var_length_read(struct tape_descriptor* drv, size_t xfer_length,
	size_t* per_xfer_len, size_t* xfer_cnt);
int setup_sg_for_read(struct tape_descriptor* drv, size_t xfer_cnt);
int lin_tape_perform_read(struct tape_descriptor** drvp, size_t* per_xfer_len,
	char* user_location, size_t xfer_length, ulong* xfer_offset,
	size_t* xfer_cnt, boolean* err, boolean* lg_rc, int* done,
	boolean* unmap);
int handle_successful_read(struct tape_descriptor* drv, char* user_location,
	size_t xfer_length, ulong* xfer_offset, size_t* xfer_cnt,
	size_t* per_xfer_len, boolean* err, boolean* lg_rc, int* done,
	boolean* unmap);
int suc_var_read_sili(struct tape_descriptor* drv, char* buffer_to_copy,
	size_t xfer_cnt, boolean* err, boolean* lg_rc, int* done);
int successful_partial_fixed_read(struct tape_descriptor* drv,
	char* user_location, size_t xfer_length, ulong* xfer_offset,
	size_t* xfer_cnt, size_t* per_xfer_len, boolean* unmap);
int handle_read_failure(struct tape_descriptor** drvp, char* user_location,
	ulong* xfer_offset, size_t xfer_cnt, boolean* err, boolean* log_rc,
	int* done, int rc);
int handle_fixed_length_read_failure(struct tape_descriptor** drvp,
	char* user_location, ulong* xfer_offset, size_t xfer_cnt, boolean* err,
	boolean* lg_rc, int* done, int rc);
int handle_var_length_read_failure(struct tape_descriptor** drvp,
	char* user_location, ulong* xfer_offset, size_t xfer_cnt, boolean* err,
	boolean* lg_rc, int* done, int rc);
ssize_t lin_tape_drive_write(struct tape_descriptor**, struct file*,
	const char*, size_t, loff_t*);
int lin_tape_drive_write_check(struct tape_descriptor** drvp);
int lin_tape_write_validate_parms(struct tape_descriptor* drv,
	size_t xfer_length, size_t* per_xfer_len, size_t* xfer_cnt);
int lin_tape_write_check_eom(struct tape_descriptor* drv);
int lin_tape_direct_or_buffered(struct tape_descriptor* drv,
	const char* user_location, size_t xfer_cnt, char rw,
	boolean verify_read, boolean* pg_mapd);
int lin_tape_perform_write(struct tape_descriptor** drvp, size_t* per_xfer_len,
	size_t* xfer_cnt, ulong* xfer_offset, const char* user_location,
	size_t xfer_length, boolean* done);
int handle_successful_write(struct tape_descriptor* drv, size_t* per_xfer_len,
	size_t* xfer_cnt, ulong* xfer_offset, size_t xfer_length,
	const char* user_location, unchar* done);
int handle_write_failure(struct tape_descriptor* drv, size_t* xfer_cnt,
	ulong* xfer_offset, unchar* done, int rc);
int lin_tape_flush(struct tape_descriptor**, struct file*);
int tape_rw_buffer(struct tape_descriptor*, size_t);
int tape_copy_write_buffer(struct tape_descriptor*, const char*, size_t, boolean fpfo);
int tape_copy_read_buffer(struct tape_descriptor*, char*, size_t, boolean fpfo);
int tape_init_sg(struct tape_descriptor*, size_t);
int tape_build_sg(struct tape_descriptor*, size_t);
int tape_ext_sg(struct tape_descriptor*, size_t);
int tape_del_sg(struct tape_descriptor*);
int lin_tape_get_user_pages(struct tape_descriptor *, const char*,
	size_t, char);
int lin_tape_release_user_pages(struct tape_descriptor *, char);
char char_to_hex(char);
boolean is_hex(char);
/* Busy flags */
#define F_ADD_REMOVE (1)
#define F_SEND      (1 << 1)
#define F_OPEN      (1 << 2)
#define F_CLOSE     (1 << 3)
#define F_SCAN      (1 << 4)
#define F_READ      (1 << 5)
#define F_WRITE     (1 << 6)
#define F_IOCTL     (1 << 7)
void set_drive_open(struct tape_descriptor* drv, boolean v, boolean lock_held);
void set_changer_open(struct changer_descriptor* chg, boolean v,
	boolean lock_held);
void printk_hex(char* sz, int len, int line);
int lin_tape_pass_thru_from_pfoID(struct tape_descriptor **drvp,
				unchar* cdb,
				unchar cmd_length,
				unchar* buffer,
				uint *buffer_length,
				uint data_direction,
				uint timeout,
				unchar* sense,
				unchar *sense_length,
				unchar *sense_avail,
				int *resid,
				int *errors);
int lin_tape_changer_pass_thru_from_pfoID(struct changer_descriptor **chgp,
				unchar* cdb,
				unchar cmd_length,
				unchar* buffer,
				uint *buffer_length,
				uint data_direction,
				uint timeout,
				unchar* sense,
				unchar *sense_length,
				unchar *sense_avail,
				int *resid,
				int *errors);

#endif
