/*******************************************************************************
 *   lin_tape_scsi_trace.c
 *
 *   (C) Copyright International Business Machines  Corp., 2007-2020
 *   This file is part of the IBM Linux Enhanced SCSI Tape and
 *   Medium Changer Device Driver.
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ******************************************************************************/

#include "lin_tape_tape.h"
extern int lin_tape_debug;

/*******************************************************************************
*                                                                              *
*  NAME: tape_log_msg                                                          *
*                                                                              *
*  PURPOSE: Add a one-byte message to log and increase log length              *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    log: the log to which to add the messages                                 *
*    log_len: (input/output) the current length of valid data in the log       *
*    msg: the byte to log                                                      *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void tape_log_msg(unchar *log, uint *log_len, unchar msg)
{
	log_multibyte_msg(log, log_len, 1, msg, 0, 0, 0, 0);
	return;
} /* tape_log_msg() */

/*******************************************************************************
*                                                                              *
*  NAME: tape_log_msg2                                                         *
*                                                                              *
*  PURPOSE: Add two one-byte messages to log and increase log length           *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    log: the log to which to add the messages                                 *
*    log_len: (input/output) the current length of valid data in the log       *
*    msg1: the first byte                                                      *
*    msg2: the second byte                                                     *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void tape_log_msg2(unchar *log, uint *log_len, unchar msg1, unchar msg2)
{
	log_multibyte_msg(log, log_len, 2, msg1, msg2, 0, 0, 0);
	return;

} /* tape_log_msg2() */

/*******************************************************************************
*                                                                              *
*  NAME: tape_log_msg3                                                         *
*                                                                              *
*  PURPOSE: Add three one-byte messages to log and increase log length         *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    log: the log to which to add the messages                                 *
*    log_len: (input/output) the current length of valid data in the log       *
*    msg1: the first byte                                                      *
*    msg2: the second byte                                                     *
*    msg3: the third byte                                                      *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void tape_log_msg3(unchar *log, uint *log_len, unchar msg1, unchar msg2,
	unchar msg3)
{
	log_multibyte_msg(log, log_len, 3, msg1, msg2, msg3, 0, 0);
	return;

} /* tape_log_msg3() */

/*******************************************************************************
*                                                                              *
*  NAME: tape_log_msg4                                                         *
*                                                                              *
*  PURPOSE: Add four one-byte messages to log and increase log length          *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    log: the log to which to add the messages                                 *
*    log_len: (input/output) the current length of valid data in the log       *
*    msg1: the first byte                                                      *
*    msg2: the second byte                                                     *
*    msg3: the third byte                                                      *
*    msg4: the fourth byte                                                     *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void tape_log_msg4(unchar *log, uint *log_len, unchar msg1, unchar msg2,
	unchar msg3, unchar msg4)
{
	log_multibyte_msg(log, log_len, 4, msg1, msg2, msg3, msg4, 0);
	return;

} /* tape_log_msg4() */

/*******************************************************************************
*                                                                              *
*  NAME: tape_log_msg5                                                         *
*                                                                              *
*  PURPOSE: Add five one-byte messages to log and increase log length          *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    log: the log to which to add the messages                                 *
*    log_len: (input/output) the current length of valid data in the log       *
*    msg1: the first byte                                                      *
*    msg2: the second byte                                                     *
*    msg3: the third byte                                                      *
*    msg4: the fourth byte                                                     *
*    msg5: the fifth byte                                                      *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void tape_log_msg5(unchar *log, uint *log_len, unchar msg1, unchar msg2,
	unchar msg3, unchar msg4, unchar msg5)
{
	log_multibyte_msg(log, log_len, 5, msg1, msg2, msg3, msg4, msg5);
	return;

} /* tape_log_msg5() */

/*******************************************************************************
*                                                                              *
*  NAME: log_multibyte_msg                                                     *
*                                                                              *
*  PURPOSE: Add up to five one-byte messages to log and increase log length    *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    log: the log to which to add the messages                                 *
*    log_len: (input/output) the current length of valid data in the log       *
*    num_msgs: the number of valid messages in the following parameters        *
*    msg1: the first byte                                                      *
*    msg2: the second byte                                                     *
*    msg3: the third byte                                                      *
*    msg4: the fourth byte                                                     *
*    msg5: the fifth byte                                                      *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void log_multibyte_msg(unchar* log, uint* log_len, int num_msgs, unchar msg1,
	unchar msg2, unchar msg3, unchar msg4, unchar msg5)
{
	int i = 0, j = 0;
	boolean full = False;

	if(!log) {
		*log_len = 0;
		return;
	} /* if */

	for(i = 1; i <= num_msgs && !full; i++) {
		if((*log_len + i) >= MaxMessageLogSize) {
				full = True;
				for(j = i; j > 0; j--)
					log[MaxMessageLogSize - j] = TRC_TRUNC;
				*log_len = MaxMessageLogSize;
		} /* if */
	} /* for */

	if(!full) {
		if(num_msgs >= 1) log[*log_len] = msg1;
		if(num_msgs >= 2) log[*log_len + 1] = msg2;
		if(num_msgs >= 3) log[*log_len + 2] = msg3;
		if(num_msgs >= 4) log[*log_len + 3] = msg4;
		if(num_msgs >= 5) log[*log_len + 4] = msg5;
		*log_len += num_msgs;
	} /* if */

	return;

} /* log_multibyte_msg() */

/*******************************************************************************
*                                                                              *
*  NAME: tape_setup_and_log_full                                               *
*                                                                              *
*  PURPOSE: Fill out trace structure and log trace                             *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    drv: tape drive for which log entry is being written                      *
*    path: the device index of the path we are logging                         *
*    code: the code to log                                                     *
*    addtl_buf: buffer containing code-specific data                           *
*    addtl_len: length of addtl_buf in bytes                                   *
*                                                                              *
*  RETURNS: 0 on success or error code                                         *
*                                                                              *
*******************************************************************************/

int tape_setup_and_log_full(struct tape_descriptor* drv, int path, int code,
	char* addtl_buf, int addtl_len)
{
	int rc = 0, len = addtl_len + 6;
	unchar* trc = NULL;

	if(len < 0) {
		rc = -EINVAL;
		goto EXIT_LABEL;
	} /* if */

	if(len) {
		trc = kmalloc(len, GFP_KERNEL);

		if(!trc) {
			rc = -ENOMEM;
			goto EXIT_LABEL;
		} /* if */

		memset(trc, '\0', len);
	} /* if */

	if(len > 0) trc[0] = path >> 8;
	if(len > 1) trc[1] = path & 0xFF;
	if(len > 2) trc[2] = drv->host;
	if(len > 3) trc[3] = drv->channel;
	if(len > 4) trc[4] = drv->target;
	if(len > 5) trc[5] = drv->lun;

	if(len > 6) memcpy(&trc[6], addtl_buf, addtl_len);

	tape_log_full_trace(drv, code, len, trc);

EXIT_LABEL:

	if(trc) kfree(trc);
	return rc;
} /* tape_setup_and_log_full() */

/*******************************************************************************
*                                                                              *
*  NAME: tape_log_full_trace                                                   *
*                                                                              *
*  PURPOSE: Write trace information to the device's trace log                  *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    drv: tape drive for which log entry is being written                      *
*    trc_code: the code to log                                                 *
*    trc_len: length of trc in bytes                                           *
*    trc: buffer containing code-specific data                                 *
*                                                                              *
*  RETURNS: 0 on success or error code                                         *
*                                                                              *
*******************************************************************************/

void tape_log_full_trace(struct tape_descriptor *drv, unchar trc_code,
	unchar trc_len, unchar *trc)
{
	struct tape_full_trace_unit* curr = NULL;
	boolean locked = False;

	if(!drv) goto EXIT_LABEL;

	if(trc_len + 3 > FullTraceUnitSize) trc_len -= 3;

	WRITE_LOCK(&drv->full_trc_lock);
	locked = True;

	curr = drv->full_trc_tail;

	/* Check for NULL or not enough remaining space in curr */
	if(!curr || (curr->cur_len + trc_len + 3) > FullTraceUnitSize) {
		curr = kmalloc(sizeof(struct tape_full_trace_unit), GFP_KERNEL);
		if(!curr) {
			printk("lin_tape: No more memory for full trace.\n");
			goto EXIT_LABEL;
		} /* if */
		memset(curr, '\0', sizeof(struct tape_full_trace_unit));

		if(drv->full_trc_tail) drv->full_trc_tail->next = curr;
		else drv->full_trc_head = curr; /* NULL tail means NULL head */

		drv->full_trc_tail = curr;
	} /* if */

	if(trc_code == TRC_TP_ATMPT_FAILOVER) drv->failover_msgs_queued++;
	curr->unit_data[curr->cur_len] = trc_code;
	curr->cur_len++;

	if(!trc || trc_len <= 0) goto EXIT_LABEL;

	curr->unit_data[curr->cur_len] = trc_len >> 8;
	curr->cur_len++;

	curr->unit_data[curr->cur_len] = trc_len & 0xFF;
	curr->cur_len++;

	memcpy(&curr->unit_data[curr->cur_len], trc, trc_len);
	curr->cur_len += trc_len;

EXIT_LABEL:

	if(locked) WRITE_UNLOCK(&drv->full_trc_lock);
	return;

} /* tape_log_full_trace() */

/*******************************************************************************
*                                                                              *
*  NAME: tape_log_buffer_data                                                  *
*                                                                              *
*  PURPOSE: Log data in buffer up to length bytes                              *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    drv: tape drive for which log entry is being written                      *
*    length: size of buffer to log                                             *
*    buffer: data to log                                                       *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void tape_log_buffer_data(struct tape_descriptor* drv, int length,
	unchar* buffer)
{
	tape_setup_and_log_full(drv, drv->cur_path, TRC_BFF, buffer, length);
	return;
} /* tape_log_buffer_data() */

/*******************************************************************************
*                                                                              *
*  NAME: tape_log_rc                                                           *
*                                                                              *
*  PURPOSE: Log return code                                                    *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    drv: tape drive for which log entry is being written                      *
*    rc: return code to log                                                    *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void tape_log_rc(struct tape_descriptor *drv, unchar rc)
{
	tape_setup_and_log_full(drv, drv->cur_path, TRC_RC, &rc, 1);
	return;
} /* tape_log_rc() */

/*******************************************************************************
*                                                                              *
*  NAME: changer_setup_and_log_full                                            *
*                                                                              *
*  PURPOSE: Fill out trace structure and log trace                             *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    chg: medium changer for which log entry is being written                  *
*    path: the device index of the path we are logging                         *
*    code: the code to log                                                     *
*    addtl_buf: buffer containing code-specific data                           *
*    addtl_len: length of addtl_buf in bytes                                   *
*                                                                              *
*  RETURNS: 0 on success or error code                                         *
*                                                                              *
*******************************************************************************/

int changer_setup_and_log_full(struct changer_descriptor* chg, int path,
	int code, char* addtl_buf, int addtl_len)
{
	int rc = 0, len = addtl_len + 6;
	unchar* trc = NULL;

	if(len < 0) {
		rc = -EINVAL;
		goto EXIT_LABEL;
	} /* if */

	if(len) {
		trc = kmalloc(len, GFP_KERNEL);

		if(!trc) {
			rc = -ENOMEM;
			goto EXIT_LABEL;
		} /* if */

		memset(trc, '\0', len);
	} /* if */

	if(len > 0) trc[0] = path >> 8;
	if(len > 1) trc[1] = path & 0xFF;
	if(len > 2) trc[2] = chg->host;
	if(len > 3) trc[3] = chg->channel;
	if(len > 4) trc[4] = chg->target;
	if(len > 5) trc[5] = chg->lun;

	if(len > 6) memcpy(&trc[6], addtl_buf, addtl_len);

	changer_log_full_trace(chg, code, len, trc);

EXIT_LABEL:

	if(trc) kfree(trc);
	return rc;
} /* changer_setup_and_log_full() */

/*******************************************************************************
*                                                                              *
*  NAME: changer_log_full_trace                                                *
*                                                                              *
*  PURPOSE: Write trace information to the device's trace log                  *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    chg: changer for which log entry is being written                         *
*    trc_code: the code to log                                                 *
*    trc_len: length of trc in bytes                                           *
*    trc: buffer containing code-specific data                                 *
*                                                                              *
*  RETURNS: 0 on success or error code                                         *
*                                                                              *
*******************************************************************************/

void changer_log_full_trace(struct changer_descriptor *chg, unchar trc_code,
	unchar trc_len, unchar* trc)
{
	struct tape_full_trace_unit *curr = NULL;
	boolean locked = False;

	if(!chg) goto EXIT_LABEL;

	if(trc_len + 3 > FullTraceUnitSize) trc_len -= 3;

	WRITE_LOCK(&chg->full_trc_lock);
	locked = True;

	curr = chg->full_trc_tail;

	/* Check for NULL or not enough remaining space in curr */
	if(!curr || (curr->cur_len + trc_len + 3) > FullTraceUnitSize) {
		curr = kmalloc(sizeof(struct tape_full_trace_unit), GFP_KERNEL);
		if(!curr) {
			printk("lin_tape: no more memory for full trace.\n");
			goto EXIT_LABEL;
		} /* if */
		memset(curr, '\0', sizeof(struct tape_full_trace_unit));

		if(chg->full_trc_tail) chg->full_trc_tail->next = curr;
		else chg->full_trc_head = curr; /* NULL tail means NULL head */

		chg->full_trc_tail = curr;
	} /* if */

	if(trc_code == TRC_CG_ATMPT_FAILOVER) chg->failover_msgs_queued++;
	curr->unit_data[curr->cur_len] = trc_code;
	curr->cur_len++;

	if(!trc || trc_len <= 0) goto EXIT_LABEL;

	curr->unit_data[curr->cur_len] = trc_len >> 8;
	curr->cur_len++;

	curr->unit_data[curr->cur_len] = trc_len & 0xFF;
	curr->cur_len++;

	memcpy(&curr->unit_data[curr->cur_len], trc, trc_len);
	curr->cur_len += trc_len;

EXIT_LABEL:

	if(locked) WRITE_UNLOCK(&chg->full_trc_lock);
	return;

} /* changer_log_full_trace() */

/*******************************************************************************
*                                                                              *
*  NAME: changer_log_buffer_data                                               *
*                                                                              *
*  PURPOSE: Log data in buffer up to length bytes                              *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    chg: medium changer for which log entry is being written                  *
*    length: size of buffer to log                                             *
*    buffer: data to log                                                       *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void changer_log_buffer_data(struct changer_descriptor* chg, int length,
	unchar* buffer)
{
	changer_setup_and_log_full(chg, chg->cur_path, TRC_BFF, buffer, length);
	return;
} /* changer_log_buffer_data() */

/*******************************************************************************
*                                                                              *
*  NAME: changer_log_rc                                                        *
*                                                                              *
*  PURPOSE: Log return code                                                    *
*                                                                              *
*  PARAMETERS:                                                                 *
*                                                                              *
*    chg: medium changer for which log entry is being written                  *
*    rc: return code to log                                                    *
*                                                                              *
*  RETURNS: void                                                               *
*                                                                              *
*******************************************************************************/

void changer_log_rc(struct changer_descriptor* chg, unchar rc)
{
	changer_setup_and_log_full(chg, chg->cur_path, TRC_RC, &rc, 1);
	return;
} /* changer_log_rc() */
