/*******************************************************************************
 *   lin_tape_srb_tape.h
 *
 *   (C) Copyright International Business Machines  Corp., 2007-2020
 *   This file is part of the IBM Linux Enhanced SCSI Tape and
 *   Medium Changer Device Driver.
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ******************************************************************************/

#ifndef _LIN_TAPE_SRB_TAPE_H_
#define _LIN_TAPE_SRB_TAPE_H_

#define MAX_CDB_SIZE         16
#define CDB_GENERIC_6         6
#define CDB_GENERIC_10       10
#define CDB_GENERIC_12       12
#define CDB_GENERIC_16       16
#define MAX_ADDITIONAL_SENSE 80

/* command descriptor block */
union cdb {
	/* standard 6 byte */
	struct _cdb_6 {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar cmd_flags:5;
		unchar lun:3;
#elif defined (__BIG_ENDIAN)
		unchar lun:3;
		unchar cmd_flags:5;
#else
		unchar unsupported;
#endif
		unchar cmd_parms[3];
		unchar ctrl_byte;
	} cdb_6;

	/* standard 10 byte */
	struct _cdb_10 {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar cmd_flags:5;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar cmd_flags:5;
#else
		unchar unsupported;
#endif
		unchar cmd_parms[7];
		unchar ctrl_byte;
	} cdb_10;

	struct _cdb_chg_definition {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved1:5;
		unchar lun:3;
		unchar save:1;
		unchar reserved2:7;
		unchar def_parm:7;
		unchar reserved3:1;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:5;
		unchar reserved2:7;
		unchar save:1;
		unchar reserved3:1;
		unchar def_parm:7;
#else
		unchar unsupported[3];
#endif
		unchar reserved4[4];
		unchar parm_length;
		unchar ctrl_byte;
	} cdb_chg_definition;

	struct _cdb_display_msg {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved:5;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved:5;
#else
		unchar unsupported;
#endif
		unchar reserved1[2];
		unchar parm_list_length;
		unchar ctrl_byte;
	} cdb_display_msg;

	struct _cdb_erase {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar long_bit:1;
		unchar immediate:1;
		unchar reserved1:3;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:3;
		unchar immediate:1;
		unchar long_bit:1;
#else
		unchar unsupported;
#endif
		unchar reserved2[3];
		unchar ctrl_byte;
	} cdb_erase;

	struct _cdb_inquiry {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar evpd:1;
		unchar reserved1:4;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:4;
		unchar evpd:1;
#else
		unchar unsupported;
#endif
		unchar page_code;
		unchar reserved2;
		unchar alloc_length;
		unchar ctrl_byte;
	} cdb_inquiry;

	struct _cdb_load_unload {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar immediate:1;
		unchar reserved1:4;
		unchar lun:3;
		unchar reserved2[2];
		unchar load:1;
		unchar reten:1;
		unchar eot:1;
		unchar hold:1;
		unchar reserved3:4;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:4;
		unchar immediate:1;
		unchar reserved2[2];
		unchar reserved3:4;
		unchar hold:1;
		unchar eot:1;
		unchar reten:1;
		unchar load:1;
#else
		unchar unsupported[4];
#endif
		unchar ctrl_byte;
	} cdb_load_unload;

	struct _cdb_locate {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar immediate:1;
		unchar cp:1;
		unchar bt:1;
		unchar reserved1:2;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:2;
		unchar bt:1;
		unchar cp:1;
		unchar immediate:1;
#else
		unchar unsupported;
#endif
		unchar reserved3;
		unchar blk_addr[4];
		unchar reserved4;
		unchar partition;
		unchar ctrl_byte;
	} cdb_locate;

	struct _cdb_locate_16 {
		unchar op_code; /* byte 0 -- '0x92' */
#if defined __LITTLE_ENDIAN
		unchar immediate : 1; /* 1 */
		unchar cp        : 1;
		unchar reserved1 : 1;
		unchar dest_type : 2;
		unchar reserved2 : 3;

		unchar bam       : 1; /* 2 */
		unchar reserved3 : 7;
#elif defined __BIG_ENDIAN
		unchar reserved2 : 3;
		unchar dest_type : 2;
		unchar reserved1 : 1;
		unchar cp        : 1;
		unchar immediate : 1;

		unchar reserved3 : 7;
		unchar bam       : 1;
#else
		error
#endif
		unchar partition;     /* 3 */
		unchar block_id[8];   /* 4 - 11 */
		unchar reserved4[3];  /* 12 - 14 */
		unchar ctrl_byte;     /* 15 */
	} cdb_locate_16;

	struct _cdb_logselect {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar sp:1;
		unchar pcr:1;
		unchar reserved1:3;
		unchar lun:3;
		unchar reserved2:6;
		unchar pc:2;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:3;
		unchar pcr:1;
		unchar sp:1;
		unchar pc:2;
		unchar reserved2:6;
#else
		unchar unsupported[2];
#endif
		unchar reserved3[4];
		unchar parm_list_length[2];
		unchar ctrl_byte;
	} cdb_logselect;

	struct _cdb_logsense {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar sp:1;
		unchar ppc:1;
		unchar reserved1:3;
		unchar lun:3;
		unchar page_code:6;
		unchar pc:2;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:3;
		unchar ppc:1;
		unchar sp:1;
		unchar pc:2;
		unchar page_code:6;
#else
		unchar unsupported[2];
#endif
		unchar subpage_code;
		unchar reserved2[1];
		unchar parm_ptr[2];
		unchar alloc_length[2];
		unchar ctrl_byte;
	} cdb_logsense;

	struct _cdb_modeselect {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar sp:1;
		unchar reserved1:3;
		unchar pf:1;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar pf:1;
		unchar reserved1:3;
		unchar sp:1;
#else
		unchar unsupported;
#endif
		unchar reserved2[2];
		unchar parm_list_length;
		unchar ctrl_byte;
	} cdb_modeselect;

	struct _cdb_modeselect_10 {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar sp:1;
		unchar reserved1:3;
		unchar pf:1;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar pf:1;
		unchar reserved1:3;
		unchar sp:1;
#else
		unchar unsupported;
#endif
		unchar reserved2[5];
		unchar parm_list_length[2];
		unchar ctrl_byte;
	} cdb_modeselect_10;

	struct _cdb_modesense {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved1:3;
		unchar dbd:1;
		unchar reserved2:1;
		unchar lun:3;
		unchar page_code:6;
		unchar pc:2;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved2:1;
		unchar dbd:1;
		unchar reserved1:3;
		unchar pc:2;
		unchar page_code:6;
#else
		unchar unsupported[2];
#endif
		unchar reserved3;
		unchar alloc_length;
		unchar ctrl_byte;
	} cdb_modesense;

	struct _cdb_modesense_10 {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved1:3;
		unchar dbd:1;
		unchar reserved2:1;
		unchar lun:3;
		unchar page_code:6;
		unchar pc:2;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved2:1;
		unchar dbd:1;
		unchar reserved1:3;
		unchar pc:2;
		unchar page_code:6;
#else
		unchar unsupported[2];
#endif
		unchar sub_page_code;
		unchar reserved3[3];
		unchar alloc_length[2];
		unchar ctrl_byte;
	} cdb_modesense_10;

	struct _cdb_medium_removal {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved1:5;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:5;
#else
		unchar unsupported;
#endif
		unchar reserved2[2];
#if defined(__LITTLE_ENDIAN)
		unchar prevent:1;
		unchar reserved:7;
#elif defined(__BIG_ENDIAN)
		unchar reserved:7;
		unchar prevent:1;
#else
		unchar unsupported2;
#endif
		unchar ctrl_byte;
	} cdb_medium_removal;

	struct _cdb_read_blk_limits {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved1:5;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:5;
#else
		unchar unsupported;
#endif
		unchar reserved2[3];
		unchar ctrl_byte;
	} cdb_read_blk_limits;

	struct _cdb_read_buffer {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar mode:3;
		unchar reserved1:2;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:2;
		unchar mode:3;
#else
		unchar unsupported;
#endif
		unchar buf_id;
		unchar buf_offset[3];
		unchar alloc_length[3];
		unchar ctrl_byte;
	} cdb_read_buffer;

	struct _cdb_read_pos {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar service_action : 5;
		unchar obsolete : 3;
#elif defined(__BIG_ENDIAN)
		unchar obsolete : 3;
		unchar service_action : 5;
#else
		unchar unsupported;
#endif
		unchar reserved2[7];
		unchar ctrl_byte;
	} cdb_read_pos;

	struct _cdb_read_write {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar fixed:1;
		unchar sili:1;
		unchar reserved:3;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved:3;
		unchar sili:1;
		unchar fixed:1;
#else
		unchar unsupported;
#endif
		unchar xfer_length[3];
		unchar ctrl_byte;
	} cdb_read_write;

	struct _cdb_release {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar element:1;
		unchar third_party_dev_id:3;
		unchar third_party:1;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar third_party:1;
		unchar third_party_dev_id:3;
		unchar element:1;
#else
		unchar unsupported;
#endif
		unchar reserve_id;
		unchar reserved;
		unchar alloc_length;
		unchar ctrl_byte;
	} cdb_release;

	struct _cdb_report_density {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar media:1;
		unchar reserved1:4;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:4;
		unchar media:1;
#else
		unchar unsupported;
#endif
		unchar reserved2[5];
		unchar alloc_length[2];
		unchar ctrl_byte;
	} cdb_report_density;

	struct _cdb_req_sense {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved1:5;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:5;
#else
		unchar unsupported;
#endif
		unchar reserved[2];
		unchar alloc_length;
		unchar ctrl_byte;
	} cdb_req_sense;

	struct _cdb_reserve {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved1:1;
		unchar third_party_dev_id:3;
		unchar third_party:1;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar third_party:1;
		unchar third_party_dev_id:3;
		unchar reserved1:1;
#else
		unchar unsupported;
#endif
		unchar reserved[3];
		unchar ctrl_byte;
	} cdb_reserve;

	struct _cdb_rewind {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar immediate:1;
		unchar reserved1:4;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:4;
		unchar immediate:1;
#else
		unchar unsupported;
#endif
		unchar reserved2[3];
		unchar ctrl_byte;
	} cdb_rewind;

	struct _cdb_send_diagnotic {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar unit_offline:1;
		unchar device_offline:1;
		unchar self_test:1;
		unchar reserved1:1;
		unchar pf:1;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar pf:1;
		unchar reserved1:1;
		unchar self_test:1;
		unchar device_offline:1;
		unchar unit_offline:1;
#else
		unchar unsupported;
#endif
		unchar reserved2;
		unchar parm_list_length[2];
		unchar ctrl_byte;
	} cdb_send_diagnostic;

	struct _cdb_space {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar code:3;
		unchar reserved1:2;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:2;
		unchar code:3;
#else
		unchar unsupported;
#endif
		unchar count[3];
		unchar ctrl_byte;
	} cdb_space;

	struct _cdb_tur {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved1:5;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:5;
#else
		unchar unsupported;
#endif
		unchar reserved2[3];
		unchar ctrl_byte;
	} cdb_tur;

	struct _cdb_write_buffer {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar mode:3;
		unchar reserved1:2;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved1:2;
		unchar mode:3;
#else
		unchar unsupported;
#endif
		unchar buf_id;
		unchar buf_offset[3];
		unchar parm_list_length[3];
		unchar ctrl_byte;
	} cdb_write_buffer;

	struct _cdb_wfm {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar immediate:1;
		unchar write_setmarks:1;
		unchar reserved:3;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved:3;
		unchar write_setmarks:1;
		unchar immediate:1;
#else
		unchar unsupported;
#endif
		unchar count[3];
		unchar ctrl_byte;
	} cdb_wfm;

	struct _cdb_persist_reserve_in {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar serv_action:4;
		unchar reserved1:4;
#elif defined(__BIG_ENDIAN)
		unchar reserved1:4;
		unchar serv_action:4;
#else
		unchar unsupported;
#endif
		unchar reserved2[4];
		unchar alloc_length[3];
		unchar ctrl_byte;
	} cdb_persist_reserve_in;

	struct _cdb_persist_reserve_out {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar serv_action:5;
		unchar reserved1:3;
		unchar type:4;
		unchar scope:4;
#elif defined(__BIG_ENDIAN)
		unchar reserved1:3;
		unchar serv_action:5;
		unchar scope:4;
		unchar type:4;
#else
		unchar unsupported[2];
#endif
		unchar reserved2[4];
		unchar parm_list_length[2];
		unchar ctrl_byte;
	} cdb_persist_reserve_out;

	struct _cdb_pos_elem {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved:5;
		unchar lun:3;
		unchar transport_elem_addr[2];
		unchar destination_elem_addr[2];
		unchar reserved1[2];
		unchar invert:1;
		unchar reserved2:7;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved:5;
		unchar transport_elem_addr[2];
		unchar destination_elem_addr[2];
		unchar reserved1[2];
		unchar reserved2:7;
		unchar invert:1;
#else
		unchar unsupported[8];
#endif
		unchar reserved3;
	} cdb_pos_elem;

	struct _cdb_init_elem_range {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar range:1;
		unchar reserved:4;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved:4;
		unchar range:1;
#else
		unchar unsupported;
#endif
		unchar start_elem_addr[2];
		unchar reserved2[2];
		unchar elem_num[2];
		unchar reserved3[2];
	} cdb_init_elem_range;

	struct _cdb_init_elem {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved:5;
		unchar lun:3;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved:5;
#else
		unchar unsupported;
#endif
		unchar reserved1[4];
	} cdb_init_elem;

	struct _cdb_read_elem_status {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar elem_type_code:4;
		unchar vol_tag:1;
		unchar lun:3;
		unchar start_elem_addr[2];
		unchar elem_num[2];
		unchar dvcid:1;
		unchar cur_data:1;
		unchar reserved:6;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar vol_tag:1;
		unchar elem_type_code:4;
		unchar start_elem_addr[2];
		unchar elem_num[2];
		unchar reserved:6;
		unchar cur_data:1;
		unchar dvcid:1;
#else
		unchar unsupported[6];
#endif
		unchar alloc_length[3];
		unchar reserved2[2];
	} cdb_read_elem_status;

	struct _cdb_move_medium {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved:5;
		unchar lun:3;
		unchar transport_elem_addr[2];
		unchar src_addr[2];
		unchar dest_addr[2];
		unchar reserved2[2];
		unchar invert:1;
		unchar reserved3:7;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved:5;
		unchar transport_elem_addr[2];
		unchar src_addr[2];
		unchar dest_addr[2];
		unchar reserved2[2];
		unchar reserved3:7;
		unchar invert:1;
#else
		unchar unsupported[10];
#endif
		unchar reserved4;
	} cdb_move_medium;

	struct _cdb_exchange_medium {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar reserved:5;
		unchar lun:3;
		unchar transport_elem_addr[2];
		unchar src_addr[2];
		unchar first_dest_addr[2];
		unchar second_dest_addr[2];
		unchar invert1:1;
		unchar invert2:1;
		unchar reserved1:6;
#elif defined(__BIG_ENDIAN)
		unchar lun:3;
		unchar reserved:5;
		unchar transport_elem_addr[2];
		unchar src_addr[2];
		unchar first_dest_addr[2];
		unchar second_dest_addr[2];
		unchar reserved1:6;
		unchar invert2:1;
		unchar invert1:1;
#else
		unchar unsupported[10];
#endif
		unchar reserved2;
	} cdb_exchange_medium;

	struct _cdb_format_medium {
		unchar op_code;
#if defined __LITTLE_ENDIAN
		unchar immed : 1;
		unchar verify : 1;
		unchar reserved1 : 6;

		unchar format : 4;
		unchar reserved2 : 4;
#elif defined __BIG_ENDIAN
		unchar reserved1 : 6;
		unchar verify : 1;
		unchar immed : 1;

		unchar reserved2 : 4;
		unchar format : 4;
#else
		error
#endif
		unchar transfer_length[2];
		unchar control;
	} cdb_format_medium;

	struct _cdb_allow_data_overwrite {
		unchar op_code;
		unchar reserved1;
#if defined __LITTLE_ENDIAN
		unchar allow_overwrite : 4;
		unchar reserved2 : 4;
#elif defined __BIG_ENDIAN
		unchar reserved2 : 4;
		unchar allow_overwrite : 4;
#else
		error
#endif
		unchar partition;
		unchar logical_id[8];
		unchar reserved3[3];
		unchar control;
	} cdb_allow_data_overwrite;

	struct _cdb_write_dynamic_attribute {
		unchar op_code;
		unchar reserved1[5];
		unchar parameter_list_length[4];
		unchar reserved2;
		unchar control;
	} cdb_write_dynamic_attribute;

	struct _verify {
		unchar op_code;
#if defined __LITTLE_ENDIAN
		unchar fixed : 1;
		unchar bytcmp : 1;
		unchar immed : 1;
		unchar vbf : 1;
		unchar vlbpm : 1;
		unchar vte : 1;
		unchar reserved : 2;
#elif defined __BIG_ENDIAN
		unchar reserved : 2;
		unchar vte : 1;
		unchar vlbpm : 1;
		unchar vbf : 1;
		unchar immed : 1;
		unchar bytcmp : 1;
		unchar fixed : 1;
#else
		error
#endif
		unchar verify_length[3];
		unchar control;
	} verify;

	struct _cdb_rrao {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar service_action:5;
		unchar reserved:2;
		unchar uds_limits:1;
#elif defined(__BIG_ENDIAN)
		unchar uds_limits:1;
		unchar reserved:2;
		unchar service_action:5;
#else
		unchar unsupported;
#endif
		unchar list_offset[4];
		unchar alloc_length[4];
#if defined(__LITTLE_ENDIAN)
		unchar uds_type:3;
		unchar reserved1:5;
#elif defined(__BIG_ENDIAN)
		unchar reserved1:5;
		unchar uds_type:3;
#else
		unchar unsupported;
#endif
		unchar ctrl_byte;
	} cdb_rrao;

	struct _cdb_grao {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
		unchar service_action:5;
		unchar reserved:3;
		unchar process:3;
		unchar reserved1:5;
		unchar uds_type:3;
		unchar reserved2:5;
#elif defined(__BIG_ENDIAN)
		unchar reserved:3;
		unchar service_action:5;
		unchar reserved1:5;
		unchar process:3;
		unchar reserved2:5;
		unchar uds_type:3;
#else
		unchar unsupported[3];
#endif
		unchar reserved3[2];
		unchar parm_list_length[4];
		unchar reserved4;
		unchar ctrl_byte;
	} cdb_grao;

	struct _cdb_report_soc {
		unchar op_code;
#if defined(__LITTLE_ENDIAN)
        unchar service_action:5;
        unchar reserved_1:3;
        unchar reporting_opt:3;
        unchar reserved_2:4;
        unchar rctd:1;
#elif defined(__BIG_ENDIAN)
        unchar reserved_1:3;
        unchar service_action:5;
        unchar rctd:1;
        unchar reserved_2:4;
        unchar reporting_opt:3;
#else
		unchar unsupported[2];
#endif
        unchar requested_opcode;
        unchar requested_service_action[2];
        unchar allocation_length[4];
        unchar reserved_3;
        unchar control;
	} cdb_report_soc;

};


#define UDS_LIMITS_PAGE  4
#define RAO_SERVICE_ACTION  0x1D
#define UDS_TYPE_MASK  0x3



#define MAX_VOLUME_LABEL_SIZE 20
#define CommandDeferred       0x71

struct tape_sense_data {
#if defined(__LITTLE_ENDIAN)
	unchar err_code:7;
	unchar valid:1;
	unchar seg_num;
	unchar sense_key:4;
	unchar reserved1:1;
	unchar ili:1;
	unchar eom:1;
	unchar fm:1;
	unchar information[4];
	unchar additional_sense_len;
	unchar cmd_spec[4];
	unchar asc;
	unchar ascq;
	unchar field_replaceable;
	unchar bit_ptr:3;
	unchar bpv:1;
	unchar reserved2:2;
	unchar cd:1;
	unchar sksv:1;
	unchar sksv_field_ptr;
	unchar sense_key_spec;
#elif defined(__BIG_ENDIAN)
	unchar valid:1;
	unchar err_code:7;
	unchar seg_num;
	unchar fm:1;
	unchar eom:1;
	unchar ili:1;
	unchar reserved1:1;
	unchar sense_key:4;
	unchar information[4];
	unchar additional_sense_len;
	unchar cmd_spec[4];
	unchar asc;
	unchar ascq;
	unchar field_replaceable;
	unchar sksv:1;
	unchar cd:1;
	unchar reserved2:2;
	unchar bpv:1;
	unchar bit_ptr:3;
	unchar sksv_field_ptr;
	unchar sense_key_spec;
#else
	unchar unsupported[18];
#endif

/* make the length to be 24 so that the data structure is not paded */
	unchar vendor_specific1;
	unchar vendor_specific2;
	unchar vendor_specific3;
	unchar vendor_specific4;
	unchar vendor_specific5;
	unchar vendor_specific6;
	unchar vendor_specific_sense[MAX_ADDITIONAL_SENSE - 6];
};

#ifndef VERIFY_TAPE_DATA
#define VERIFY_TAPE_DATA (0x13)
#endif

#ifndef SEEK_16
#define SEEK_16 (0x92)
#endif

#ifndef LOCATE_16
#define LOCATE_16 SEEK_16
#endif

#ifndef ALLOW_DATA_OVERWRITE
#define ALLOW_DATA_OVERWRITE (0x82)
#endif

#endif
