/*******************************************************************************
 *   lin_tape_support_list.h
 *
 *   (C) Copyright International Business Machines  Corp., 2007-2020
 *   This file is part of the IBM Linux Enhanced SCSI Tape and
 *   Medium Changer Device Driver.
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ******************************************************************************/

#ifndef _LIN_TAPE_SUPPORT_LIST_H_
#define _LIN_TAPE_SUPPORT_LIST_H_

#include "IBM_tape.h"

static
char *supported_vendor[] = { "IBM", "QUANTUM", "ADIC", "HP", NULL };

static
char *supported_device[] = {
	/* Keep this array in sync with supported_devid array */
	"03590",                /* 1. Magstar Drive  */
	"03592",                /* 2. Jaguar Drive   */
	"ULTRIUM-TD1",          /* 3. Ultrium Gen 1  */
	"ULT3580-TD1",          /* 4. Ultrium Gen 1  */
	"ULTRIUM-TD2",          /* 5. Ultrium Gen 2  */
	"ULT3580-TD2",          /* 6. Ultrium Gen 2  */
	"ULTRIUM-TD3",          /* 7. Ultrium Gen 3  */
	"ULT3580-TD3",          /* 8. Ultrium Gen 3  */
	"ULTRIUM-HH3",          /* 9. Ultrium Gen 3 Half-High */
	"ULT3580-HH3",          /* 10. Ultrium Gen 3 Half-High */
	"ULTRIUM-TD4",          /* 11. Ultrium Gen 4 */
	"ULT3580-TD4",          /* 12. Ultrium Gen 4 */
	"ULTRIUM-HH4",          /* 13. Ultrium Gen 4 Half-High  */
	"ULT3580-HH4",          /* 14. Ultrium Gen 4 Half-High  */
	"ULTRIUM-TD5",          /* 15. Ultrium Gen 5 */
	"ULT3580-TD5",          /* 16. Ultrium Gen 5 */
	"ULTRIUM-HH5",          /* 17. Ultrium Gen 5 Half-High  */
	"ULT3580-HH5",          /* 18. Ultrium Gen 5 Half-High  */
	"HH LTO Gen 5",         /* 19. Ultrium Gen 5 Half-High  */
	"ULTRIUM-TD6",          /* 20. Ultrium Gen 6 */
	"ULT3580-TD6",          /* 21. Ultrium Gen 6 */
	"ULTRIUM-HH6",          /* 22. Ultrium Gen 6 Half-High  */
	"ULT3580-HH6",          /* 23. Ultrium Gen 6 Half-High  */
	"HH LTO Gen 6",         /* 24. Ultrium Gen 6 Half-High  */
	"ULTRIUM-TD7",          /* 25. Ultrium Gen 7 */
	"ULT3580-TD7",          /* 26. Ultrium Gen 7 */
	"ULTRIUM-HH7",          /* 27. Ultrium Gen 7 Half-High  */
	"ULT3580-HH7",          /* 28. Ultrium Gen 7 Half-High  */
	"HH LTO Gen 7",         /* 29. Ultrium Gen 7 Half-High  */
	"ULTRIUM-TD8",          /* 30. Ultrium Gen 8 */
	"ULT3580-TD8",          /* 31. Ultrium Gen 8 */
	"ULTRIUM-HH8",          /* 32. Ultrium Gen 8 Half-High  */
	"ULT3580-HH8",          /* 33. Ultrium Gen 8 Half-High  */
	"HH LTO Gen 8",         /* 34. Ultrium Gen 8 Half-High  */
	"ULTRIUM-TD9",          /* 35. Ultrium Gen 9 */
	"ULT3580-TD9",          /* 36. Ultrium Gen 9 */
	"ULTRIUM-HH9",          /* 37. Ultrium Gen 9 Half-High  */
	"ULT3580-HH9",          /* 38. Ultrium Gen 9 Half-High  */
	"HH LTO Gen 9",         /* 39. Ultrium Gen 9 Half-High  */
	"ULTRIUM 9-SCSI",       /* 39. Ultrium Gen 9 Half-High  */
	"ULT3581-TA",           /* 40. IBM Autoloader */
	"ULT3581-TA2",          /* 41. 2U Autoloader  */
	"ULT3582-TL",           /* 42. 3582 Library   */
	"ULT3583-TL",           /* 43. 3583 Library   */
	"03584L",               /* 44. 3584 Library   */
	"3576-MTL",             /* 45. 3576 Library   */
	"3573-TL",              /* 46. 2U4U Library   */
	"3577",                 /* 47. 5U library     */
	"3572",                 /* 48. 1U library     */
	"Scalar i500",          /* 49. Quantum Library*/
	"Scalar i6000",         /* 50. Quantum Library*/
	"Scalar i3-i6",         /* 51. Quantum Library*/
	"ESL G3 Series",        /* 42. HP Library*/
	"MSL6480",              /* 53. HP Library*/
	"1x8 G2 AUTOLDR",       /* 54. HP Library*/
	"MSL3040",              /* 55. HP Library*/
	"Ultrium 7-SCSI",       /* 56. HP Ultrium Gen 7 Half-High */
	"Ultrium 8-SCSI",       /* 57. HP Ultrium Gen 8 Half-High */
	"3555-TL",              /* 58. 3U Library   */
	NULL
};

static
int supported_devid[] = {
	/* Keep this array in sync with supported_device array */
	IBM_3590,   /* 1 */
	IBM_3592,   /* 2 */
	IBM_3580,   /* 3 */
	IBM_3580,   /* 4 */
	IBM_3580,   /* 5 */
	IBM_3580,   /* 6 */
	IBM_3580,   /* 7 */
	IBM_3580,   /* 8 */
	IBM_3580,   /* 9 */
	IBM_3580,   /* 10 */
	IBM_3580,   /* 11 */
	IBM_3580,   /* 12 */
	IBM_3580,   /* 13 */
	IBM_3580,   /* 14 */
	IBM_3580,   /* 15 */
	IBM_3580,   /* 16 */
	IBM_3580,   /* 17 */
	IBM_3580,   /* 18 */
	IBM_3580,   /* 19 */
	IBM_3580,   /* 20 */
	IBM_3580,   /* 21 */
	IBM_3580,   /* 22 */
	IBM_3580,   /* 23 */
	IBM_3580,   /* 24 */
	IBM_3580, /* 25 */
	IBM_3580, /* 26 */
	IBM_3580, /* 27 */
	IBM_3580, /* 28 */
	IBM_3580, /* 29 */
	IBM_3580, /* 30 */
	IBM_3580, /* 31 */
	IBM_3580, /* 32 */
	IBM_3580, /* 33 */
	IBM_3580, /* 34 */
    IBM_3580, /* 35 */
	IBM_3580, /* 36 */
    IBM_3580, /* 37 */
	IBM_3580, /* 38 */
    IBM_3580, /* 39 */
    IBM_3580, /* 39 */
	IBM_3581, /* 40 */
	BDT_3581, /* 41 */
	IBM_3582, /* 42 */
	IBM_3583, /* 43 */
	IBM_3584, /* 44 */
	IBM_3576, /* 45 */
	IBM_3573, /* 46 */
	IBM_3577, /* 47 */
	IBM_3572, /* 48 */
	IBM_3576, /* 49 */
	IBM_3576, /* 50 */
	IBM_3576, /* 51 */
	IBM_3576, /* 52 */
	IBM_3576, /* 53 */
	IBM_3576, /* 54 */
	IBM_3576, /* 55 */
	IBM_3580, /* 56 */
	IBM_3580, /* 57 */
	IBM_3573  /* 58 */
};

#endif
