/*******************************************************************************
 *   lin_tape_timeout_tape.h
 *
 *   (C) Copyright International Business Machines  Corp., 2007-2020
 *   This file is part of the IBM Linux Enhanced SCSI Tape and
 *   Medium Changer Device Driver.
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ******************************************************************************/

#ifndef _LIN_TAPE_TIMEOUT_TAPE_H_
#define _LIN_TAPE_TIMEOUT_TAPE_H_

#define EraseTimeOut             0
#define InquiryTimeOut           1
#define LoadTimeOut              2
#define LocateTimeOut            3
#define LogSelectTimeOut         4
#define LogSenseTimeOut          5
#define ModeSelectTimeOut        6
#define ModeSenseTimeOut         7
#define UnloadTimeOut            8
#define PreventAllowMediaTimeOut 9
#define ReadTimeOut             10
#define ReadBlockLimitsTimeOut  11
#define ReadBufferTimeOut       12
#define ReadPositionTimeOut     13
#define ReleaseTimeOut          14
#define ReportDensityTimeOut    15
#define ReserveTimeOut          16
#define RewindTimeOut           17
#define SpaceTimeOut            18
#define TestUnitReadyTimeOut    19
#define WriteTimeOut            20
#define WriteFileMarkTimeOut    21
#define DisplayMessageTimeOut   22
#define SendDiagnosticTimeOut   23
#define ChangeDefinitionTimeOut 24
#define DefaultTimeOut          25
#define PersistReserveInTimeOut 26
#define PersistReserveOutTimeOut 27
#define FormatMediumTimeOut     28
#define AllowOverwriteTimeout   29
#define WriteDynamicRTTimeout   30
#define RAOTimeout              31

#define DefaultTimeOutValue     1000

#define compute_timeout(DevID, CommandTimeOut) \
        (((drv->autotimeout)? (drv->timeout[opCodes[CommandTimeOut]]): \
	    (DevID == IBM_3590)  ? (IBM3590TimeOut[CommandTimeOut]) : \
		(DevID == IBM_3592)  ? (IBM3592TimeOut[CommandTimeOut]) : \
		(DevID == IBM_3580)  ? (IBM3580TimeOut[CommandTimeOut]) : \
		(DefaultTimeOutValue) ) * HZ )

static uint IBM3580TimeOut[] = {
	121500, // Erase
	180,   // Inquiry
	1100,  // Load
	31000, // Locate
	90,    // LogSelect
	90,    // LogSense
	300,   // ModeSelect
	90,    // ModeSense
	1100,  // Unload
	90,    // PreventAllowMedia
	2000,  // Read
	90,    // ReadBlockLimites
	720,   // ReadBuffer
	90,    // ReadPosition
	90,    // Release
	90,    // ReportDensity
	90,    // Reserve
	900,   // Rewind
	50000, // Space
	90,    // TestUnitReady
	2000,  // Write
	1800,  // WriteFileMark
	0,     // DisplayMessage
	3200,  // SendDiagnosic
	60,    // ChangeDefinition
	500,   // Default
	90,    // Persistent Reserve In
	90,    // Persistent Reserve Out
	4000,  // Format medium
	75,    // Allow Data Overwrite
	75,    // Write Dynamic Runtime Attribute
	0      // RAO not valid on 3580
};

static ushort IBM3590TimeOut[] = {
	5600,                   /* Erase                    */
	60,                     /* Inquiry                  */
	850,                    /* Load                     */
	5600,                   /* Locate                   */
	60,                     /* LogSelect                */
	60,                     /* LogSense                 */
	4000,                   /* ModeSelect               */
	360,                    /* ModeSense                */
	850,                    /* Unload                   */
	60,                     /* PreventAllowMedia        */
	850,                    /* Read                     */
	60,                     /* ReadBlockLimites         */
	350,                    /* ReadBufferTimeOut        */
	60,                     /* ReadPosition             */
	90,                     /* Release                  */
	60,                     /* ReportDensity            */
	850,                    /* Reserve                  */
	850,                    /* Rewind                   */
	5600,                   /* Space                    */
	60,                     /* TestUnitReady            */
	850,                    /* Write                    */
	850,                    /* WriteFileMark            */
	200,                    /* DisplayMessage           */
	850,                    /* SendDiagnostic           */
	60,                     /* ChangeDefinition         */
	500,                    /* Default                  */
	60,                     /* Persistent Reserve In    */
	1080,                   /* Persistent Reserve Out   */
	2100,                   /* Format medium            */
	75,                     /* Allow Data Overwrite     */
	75,                     /* Write Dyn. Runtime Attr. */
	0      					/* RAO not valid on 3590    */
};

static ushort IBM3592TimeOut[] = {
	65000, // Erase
	180,   // Inquiry
	900,   // Load
	8400,  // Locate
	60,    // LogSelect
	60,    // LogSense
	1000,  // ModeSelect
	60,    // ModeSense
	1300,  // Unload
	60,    // PreventAllowMedia
	2400,  // Read
	60,    // ReadBlockLimites
	590,   // ReadBufferTimeOut
	60,    // ReadPosition
	60,    // Release
	60,    // ReportDensity
	1020,  // Reserve
	850,   // Rewind
	2300,  // Space
	60,    // TestUnitReady
	2400,  // Write
	1560,  // WriteFileMark
	60,    // DisplayMessage
	2320,  // SendDiagnostic
	60,    // ChangeDefinition
	500,   // Default
	60,    // Persistent Reserve In
	1080,  // Persistent Reserve Out
	3180,  // Format medium
	75,    // Allow Data Overwrite
	300,   // Write Dynamic Runtime Attribute
	35     // Recommended Access Order
};

static unchar opCodes[] = {
0x19,//EraseTimeOut
0x12,//InquiryTimeOut
0x1b,//LoadTimeOut
0x2b,//LocateTimeOut
0x4c,//LogSelectTimeOut
0x4d,//LogSenseTimeOut
0x55,//ModeSelectTimeOut
0x5a,//ModeSenseTimeOut
0x1b,//UnloadTimeOut
0x1e,//PreventAllowMediaTimeOut
0x08,//ReadTimeOut
0x05,//ReadBlockLimitsTimeOut
0x3c,//ReadBufferTimeOut
0x34,//ReadPositionTimeOut
0x57,//ReleaseTimeOut
0x44,//ReportDensityTimeOut
0x56,//ReserveTimeOut
0x01,//RewindTimeOut
0x11,//SpaceTimeOut
0x00,//TestUnitReadyTimeOut
0x0a,//WriteTimeOut
0x10,//WriteFileMarkTimeOut
0xc0,//DisplayMessageTimeOut
0x1d,//SendDiagnosticTimeOut
0x40,//ChangeDefinitionTimeOut
0x19,//DefaultTimeOut - Not used
0x5e,//PersistReserveInTimeOut
0x5f,//PersistReserveOutTimeOut
0x04,//FormatMediumTimeOut
0x82,//AllowOverwriteTimeout
0x8d,//WriteDynamicRTTimeout
0xa4//RAOTimeout
};

#endif
