#ifndef _PFO_H
#define _PFO_H
#define PFO_DATE_STR "20180322"
#define PFO_VERSION_STR "5.3.0"
/*
 * (c) Copyright 2014, 2015 Hewlett-Packard Development Company, L.P
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor
 * Boston, MA 02110-1301, USA.
 */

#include "apfo.h"
/* The intent of the following defines is that the code
 * will build for various releases of Linux.  This is how
 * we track different versions using ifdef in the source.
 */
#define AFTER_RH_62 1
#define AFTER_RH_63 1
#define AFTER_RH_64 1
#define AFTER_RH_65 1
#define AFTER_RH_66 1
#define AFTER_RH_67 1
#define AFTER_RH_6x 1
#define AFTER_RH_70 1
#define AFTER_RH_71 1
#define AFTER_RH_72 1
#define AFTER_RH_73 1

/* Misc defines needed so source is same for all versions of Linux */
#ifndef SDEV_TRANSPORT_OFFLINE
#define SDEV_TRANSPORT_OFFLINE  (SDEV_CREATED_BLOCK + 1)
#endif
#ifndef MTWEOFI
#define MTWEOFI 35
#endif

struct join_device_identifier {
	int fdev_num;
	int dev_type;
	struct file *filp;
	unsigned char *serial;
	unsigned char *wwnn;
	unsigned int host;
	unsigned int channel;
	unsigned int target;
	unsigned int lun;
};

struct join_class_interface {
	int (*join_add) (struct device *, struct class_interface *);
	void (*join_remove) (struct device *, struct class_interface *);
	int (*join_open)(struct join_device_identifier *gizmo);
	int (*join_close)(struct join_device_identifier *gizmo);
	int (*join_send)(struct join_device_identifier *gizmo,
		unchar *cdb,
		unchar cmd_length,
		unchar *buffer,
		uint buffer_length,
		uint data_direction,
		uint timeout,
		unchar *sense,
		int *sense_length,
		int *resid,
		int *result);
};
void pfo_path_change(int fdev_num, int host, int channel, int target, int lun);

#define NOT_PFO_DEVICE (-1)

int pfo_open(struct scsi_device *sdev, struct file *filp);
int pfo_close(struct scsi_device *sdev, struct file *filp);
int pfo_device_number(struct scsi_device *sdev);
int pfo_execute_req(struct scsi_device *sdev, unsigned char *cmd,
		int data_direction, void *buffer, unsigned bufflen,
		struct scsi_sense_hdr *sshdr, int timeout, int retries,
		int *resid);
int pfo_scsi_ioctl(struct scsi_device *sdev, int ioctl_cmd, void __user *arg);
int pfo_st_compat_ioctl(int fdev_plus1, int ioctl_cmd, void __user *arg);
int pfo_sg_compat_ioctl(int fdev_plus1, int ioctl_cmd, void __user *arg);
int pfo_scsi_cmd_ioctl(struct scsi_device *sdev, struct request_queue *q,
		struct gendisk *disk, fmode_t mode,
		unsigned int ioctl_cmd, void __user *arg);
int pfo_sg_scsi_ioctl(struct scsi_device *sdev, struct request_queue *q,
		struct gendisk *disk, fmode_t mode,
		struct scsi_ioctl_command __user *sic);
int pfo_st_scsi_execute(int fdev_plus1, const unsigned char *cmd,
		int data_direction, void *buffer, unsigned bufflen,
		unsigned char *raw_sense, int timeout, int retries,
		int *resid, struct rq_map_data *mdata,
		int *errors);
int pfo_sg_scsi_execute(int fdev_plus1, const unsigned char *cmd, int cmd_len,
		int data_direction, void *buffer, unsigned bufflen,
		unsigned char *raw_sense, int timeout, int retries,
		int *resid, struct rq_map_data *mdata,
		int *errors, unsigned int iov_count);
int pfo_is_sfmp_device(int pfo_device_number);

int pfo_register_sg(struct class_interface *intf,
				void (*)(struct device *, struct device *));
void pfo_unregister_sg(struct class_interface *intf);
int pfo_register_st(struct device_driver *drv,
				void (*)(struct device *, struct device *));
void pfo_unregister_st(struct device_driver *drv);
int pfo_register_sf(struct join_class_interface *intf);
void pfo_unregister_sf(void);

#endif
