#ifndef _RESOLVE_H
#define _RESOLVE_H
/**
 * This file contains the definitions needed to use resolve_ibm_hp.c.
 *
 */

/*
 * (c) Copyright 2015 Hewlett-Packard Development Company, L.P
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor
 * Boston, MA 02110-1301, USA.
 */

#include "apfo.h"

/*###############################################################*/
/* Constants used only in this file.				 */
/*###############################################################*/

/* Standard SCSI command codes */
#define INQUIRY 0x12
#define INQ_LEN 255

/* Various timeouts used for failover: */
#define PFO_TIMEOUT_10SEC (10 * HZ)
#define PFO_TIMEOUT_30SEC (30 * HZ)
#define PFO_TIMEOUT_60SEC (60 * HZ)
#define PFO_TIMEOUT_300SEC (300 * HZ)
#define PFO_COMMAND_RETRIES 10

#define SCSI_SENSE_BUFFERSIZE 256

#define PFO_RESERVE_SIZE 264
#define PFO_PREVENT_SIZE 4104
#define PFO_PERSIST_SIZE 8204

#define PFO_NO_PATH_SELECTED (-1)
#define PFO_EINVAL (-22)

#define TAPE_DEVICE 1
#define MEDIA_CHANGER 8

/*###############################################################*/
/* Data structures used only in this file.			 */
/*###############################################################*/
struct gen_path_entry {
	int path_number;
	int fdev_number;
	char is_active_mc_path;
	int link_speed;
	unsigned char wwpn[WWN_SIZE];
};

struct failover_dev {
	int fo_dev_number;
	volatile int path_with_session;
	int fsc_sequence_num;
	volatile int last_known_good_path;
	int previous_path_num;
	int ua_count;
	volatile int num_paths;
	int cmnd_timeout;
	struct gen_path_entry **paths;
	int marker1;
	struct apfo_scsi_command tur_cmd;
	unsigned char tur_raw_sense[SCSI_SENSE_BUFFERSIZE];
	int marker2;
	struct apfo_scsi_command save_cmd;
	unsigned char save_raw_sense[SCSI_SENSE_BUFFERSIZE];
	int marker3;
	unsigned char wwnn[WWN_SIZE];
	unsigned char pref_path_str[MAX_PATH_LEN];
	unsigned char session_key[4];
	unsigned char cached_smc_data;
	unsigned char fo_session_is_open;
	volatile char open_count;
	unsigned char cmd_retry;
	unsigned char protocol_type;
	unsigned char dev_type;
	unsigned char cmnd_byte;
	unsigned char fo_restricted;
	unsigned char drive_switch;
	unsigned char force_return_sense_to_user;
	unsigned char fo_enabled;
	unsigned char open_fail_sent;
	unsigned char ncn_just_executed;
	unsigned char paths_lost_on_fo_command_retry;
	unsigned char sfmp_device;
	int marker4;
	unsigned char smc_prevent_allow[PFO_PREVENT_SIZE];
	unsigned char smc_persistent_data[PFO_PERSIST_SIZE];
	unsigned char smc_reserve_data[PFO_RESERVE_SIZE];
};

/* Function prototypes */
int resolve_get_wwnn_from_path(int path_num, unsigned char wwnn[]);
int resolve_print_sessions(int fdev_num, char *string, int size);
int resolve_add_new_path(int path_num, int *fdev_num,
			unsigned char *wwnn, unsigned char *serial);
int resolve_remove_path(int path_num);
int resolve_ctrl_get(char *out_string, size_t size);
int resolve_ctrl(int fdev_num, const char *in_string, size_t size);
int resolve_print_device(int fdev_num, char *string, int size);
void resolve_exit(void);

/* Definitions of functions provided by decide.c */
int decide_send(int fdev, struct apfo_scsi_command *cmd);
int decide_open(int fdev);
int decide_close(int fdev);
int decide_add_new_path(int path, int *fdev,
			unsigned char *wwnn, unsigned char *serial);
int decide_remove_path(int path_num);
int decide_print_device(int fdev, char *string, int size);
int decide_print_sessions(int fdev, char *string, int size);
int decide_ctrl(int fdev, const char *string, size_t count);
int decide_ctrl_get(char *string, size_t count);
void decide_exit(void);
int decide_get_wwnn_from_fdev(int fdev_num, unsigned char wwnn[]);
int decide_get_wwnn_from_path(int path_num, unsigned char wwnn[]);
int decide_failover_is_enabled(int path_num,
 				int device_type,
 				int *active_mc_path);

#endif
